% Przyklad 2.11: 
% m-plik: przyklad_2_11.m
%
% Cyfrowe przetwarzanie sygnalow. Podstawy, multimedia, transmisja. PWN, Warszawa, 2014.
% Rozdzial 2. Podstawy analizy czestotliwosciowej i probkowanie sygnalow. 
% Autorzy rozdzialu: Przemyslaw Korohoda, Krzysztof Duda.
%
% Wiecej na stronie internetowej: http://teledsp.kt.agh.edu.pl
%
% Opracowanie przykladu: kwiecien 2013 r./KD.

close all;  clear all;  clc;

fp= 100;    % czestotliwosc probkowania w Hz;
N = 256;    % liczba probek;
A = 3;      % amplituda sygnalu;
f = 3;      % czestotliwosc sygnalu analogowego w Hz;
p = 0.1;    % faza w rad;
D = 0.1;    % tlumienie;
dt= 1/(fp);
t = (0:N-1)*dt;

%% sygnal testowy:
x = A*cos(2*pi*f*t+p).*exp(-D*t);

%% BY1 bez korekcji przecieku widmowego:
NI=0; [Ome, de, Ae, pe] = BY1_LC(x,NI);
fe = Ome*fp/(2*pi);     % przeliczenie modelu;
De = de*fp;             % przeliczenie modelu;
blad(1,:) = [fe-f, De-D, Ae-A, pe-p];

%% BY1 korekcja przecieku widmowego 5 iteracji:
NI=5; [Ome, de, Ae, pe] = BY1_LC(x,NI);
fe = Ome*fp/(2*pi);     % przeliczenie modelu;
De = de*fp;             % przeliczenie modelu;
blad(2,:) = [fe-f, De-D, Ae-A, pe-p];
%% 
blad_f_D_A_p = num2str(blad),

%% Rysunki
%% DFT
Xd = fft(x); 
[val ind] = max(abs(Xd(1:round(N/2))));
dw = (2*pi/N);  
wd = (0:N-1)*dw;
fd = fp*wd/(2*pi);
%% DTFT 
Nfft = 2^16;
Xc = fft(x,Nfft);  % zageszczone probkowanie widma sygnalu dyskretnego przez dolaczenie wektora zer do dlugosci sygnalu Nfft przed liczeniem DFT
[val ind] = max(abs(Xc(1:round(Nfft/2))));
dwc= (2*pi/Nfft);  
wdc= (0:Nfft-1)*dwc;
fdc= fp*wdc/(2*pi);

FontSize=16;
FontName   = 'Times New Roman';
figure, hold on
    plot(t,x,'.-k','LineWidth',1,'MarkerSize',12), 
    title('x(t)= 3cos(2\pi3t+0.1)exp(-0.1t)','FontSize',FontSize,'FontName',FontName)
    xlabel('t_n [s]','FontSize',FontSize,'FontName',FontName)
    ylabel('x(t_n)','FontSize',FontSize,'FontName',FontName)
    axis tight,box on       
    v=axis; axis([v(1) v(2) -3.01 3.01])
    set(gca,'FontSize',FontSize,'FontName',FontName) 
figure, hold on
    H1=stem(fd, abs(Xd),'ob','filled');  set(H1,'LineWidth',1);
    plot(fdc,abs(Xc),'-r','LineWidth',1),     
    H2=stem(fe, 400, '--k','filled');   set(H2,'LineWidth',2);
    plot(fdc,abs(Xc),'-r','LineWidth',1),             
    legend('|DFT|','|DTFT|')
    xlabel('f [Hz]','FontSize',FontSize,'FontName',FontName)    
    axis tight,box on        
    set(gca,'FontSize',FontSize,'FontName',FontName) 
    v=axis; axis([2 6 v(3) 350])

% KONIEC PRZYKLADU 2.11.