% Przyklad 2.12: 
% m-plik: przyklad_2_12.m
%
% Cyfrowe przetwarzanie sygnalow. Podstawy, multimedia, transmisja. PWN, Warszawa, 2014.
% Rozdzial 2. Podstawy analizy czestotliwosciowej i probkowanie sygnalow. 
% Autorzy rozdzialu: Przemyslaw Korohoda, Krzysztof Duda.
%
% Wiecej na stronie internetowej: http://teledsp.kt.agh.edu.pl
%
% Opracowanie przykladu: kwiecien 2013 r./PK.

close all;  clear all;  clc;

N=2^13;       fpr=1000;         % dlugosc ciagu i czestotliwosc probkowania w [Hz];
dt=1/fpr;     t=0:dt:(N-1)*dt;  % os czasu w [s];
[b1,a1]=butter(15,0.6);         % modelujemy kanal jako filtr Butterwortha (patrz rozdzial 4);
[H,W]=freqz(b1,a1,5000);        % wyznaczamy charakterystyki kanalu transmisyjnego; 
F=W/(2*pi);   f=fpr*F;          % przeliczamy czestotliwosci;

%% definiujemy sygnal modulujacy (xm) oraz nosna (xc):
fm=2;         fc=300;           % przyjmujemy czestotliwosci sygnalu modulujacego i nosnej w [Hz];
xm=sinc(fm*(t-(N/2)*dt));       % sygnal modulujacy;
xc=cos(2*pi*fc*t);              % sygnal nosnej;
x=xm.*xc;                       % sygnal zmodulowany;

%% "przesylamy" sygnal przez modelowany cyfrowo kanal transmisyjny:
y=filter(b1,a1,x);
P=unwrap(angle(H));     df=f(2)-f(1);       % wyliczamy przesuniecie fazy i przyrost czestotliwosci;
k1=find((f>fc-df/2)&(f<=fc+df/2));          % wyznaczamy indeks dla f=fc;
A=abs(H(k1));                               % zmiana amplitudy sygnalu dla f=fc;
tp=-P(k1)/(2*pi*fc),                        % opoznienie fazowe w [s] wedlug wzoru;
tg=-(1/(2*pi))*(P(k1+1)-P(k1-1))/(2*df),    % opoznienie grupowe w [s] wedlug wzoru;
tg_vs_dt=tg/dt,                             % proporcja tg do kroku probkowania na osi czasu;
tp_vs_dt=tp/dt,                             % proporcja tp do kroku probkowania na osi czasu;

%% na podstawie wyliczonych wartosci opoznien konstruujemy sygnal porownawczy:
xm_test=sinc(fm*(t-(N/2)*dt-tg));           % opozniony sygnal modulujacy;
xc_test=cos(2*pi*fc*(t-tp));                % opozniony sygnal nosnej;
y_test=A*xc_test.*xm_test;                  % sygnal zmodulowany po transmisji;
x_err=y_test-y;                             % sygnal roznicy (bledu);
max_abs_err=max(abs(x_err)),                % maksymalny blad bezwzgledny;
   
    figure(1);   % fragment charakterystyki amplitudowej filtra modelujacego kanal transmisyjny;
        k=k1-20:k1+20;
        plot(f(k),abs(H(k)),'b.-');   grid on;   hold on;  axis tight;
        plot([f(k(1)),f(k(end))],[abs(H(k(1))),abs(H(k(end)))],'k');
            xlabel('f [Hz]');    title('Fragment ch-ki amplitudowej kanalu w rejonie f nosnej');
    
    figure(2);   % graficzne porownanie sygnalow;
        plot(t,y,'b.-');  grid on;  hold on;
        plot(t,y_test,'r.');
            legend('y(t)','y_t_e_s_t(t)');    xlabel('t [s]');  title('Graficzne porownanie sygnalow (prosze uzyc "zoom")');
            axis tight;
            % sygnaly sa bardzo podobne, ale nie identyczne - dlaczego?
        
% KONIEC PRZYKLADU 2.12.