% Przyklad 2.6: 
% m-plik: przyklad_2_6.m
%
% Cyfrowe przetwarzanie sygnalow. Podstawy, multimedia, transmisja. PWN, Warszawa, 2014.
% Rozdzial 2. Podstawy analizy czestotliwosciowej i probkowanie sygnalow. 
% Autorzy rozdzialu: Przemyslaw Korohoda, Krzysztof Duda.
%
% Wiecej na stronie internetowej: http://teledsp.kt.agh.edu.pl
%
% Opracowanie przykladu: kwiecien 2013 r./PK.

clc;       clear;    close all;

dt=0.0001; tmax=0.5; t=-tmax:dt:tmax;   % odcinek pseudo-ciaglej osi czasu;
t=t(:);                                 % ulozenie elementow ciagu t w wektorze kolumnowym;

Dt=0.0025; fpr=1/Dt;                    % Dt to faktyczny odstep probkowania;
tn0=0:Dt:tmax;  tn=[-tn0(end:-1:2),tn0];% ciag wartosci t dla probek, tak zeby w srodku bylo t=0;

dF=0.0001; F=-2:dF:2; F=F(:);           % odcinek pseudo-ciaglej osi F, F(:) - pionowo;
f=F*fpr;                                % odcinek pseudo-ciaglej osi czestotliwosci f, w Hz;

f0=100; t0=0.01;                        % parametry sygnalu;
x=sinc(f0*(t-t0)).^2;                   % sygnal pseudo-ciagly: x(t);
xn=sinc(f0*(tn-t0)).^2;                 % ciag probek: x[n];

NF=length(F);        H=zeros(NF,1);     % wstepne przygotowanie filtru odtwarzajacego;
H((f>-fpr/2)&(f<fpr/2))=1;              % dla wybranego zakresu f ustawiamy: H=1;
N=length(xn);        X=zeros(NF,1);     % przygotowanie do petli liczacej DTFT;
 
for n=1:N,
     n1=(n-(N+1)/2);                    % przeliczenie indeksow, by pasowaly do wzoru DTFT;
     X=X+xn(n)*exp(-j*2*pi*F*n1);
end;

X0=X.*H;                                % odtwarzamy widmo sygnalu;
Nt=length(t); x_rec=zeros(Nt,1);        % przygotowanie do petli odtwarzajacej sygnal;
fg=0.5*fpr;                             % czestotliwosc graniczna filtru odtwarzajacego;
for n=1:N,
    n1=(n-(N+1)/2);                     % przeliczenie indeksow, by pasowaly do wzoru;
    t1=t-n1*Dt;                         % przesuniecie czasu dla kazdego elementu sumy;
    x_rec=x_rec+xn(n)*(2*fg/fpr)*sinc(2*fg*t1);
end;

rec_err=max(abs(x_rec-x)),              % weryfikacja - porownanie x(t) oraz x_rec(t);

     figure(1);
         subplot(1,2,1); 
                plot(t,x,'r');          grid on;    hold on;
                plot(tn,xn,'bo');
                     xlim([-0.04,0.04]);       xlabel('t [s]','fontsize',14);    title('x(t) oraz x[n]','fontsize',14);
         subplot(1,2,2); 
                plot(f,abs(X0),'r.-');  grid on;    hold on;
                 plot(f,abs(X),'b-');
                     xlim([min(f),max(f)]);    xlabel('f [Hz]','fontsize',14);   title('Dt ^. abs(X(f)) oraz abs(X(F))','fontsize',14);
                     
     figure(2);
                plot(t,x,'r.');         grid on;    hold on;
                plot(t,x_rec,'b');
                     xlim([-0.04,0.04]);       xlabel('t [s]','fontsize',14);    title('x(t) oraz x_r_e_c(t)','fontsize',14);
 
% KONIEC PRZYKLADU 2.6.