function [h, a]=fir_mse_W(M,Omc,Wp,Ws);
% [h,a]=fir_mse_W(M,Omc,Wp,Ws);
%
% projektowanie optymalnego w sensie sredniokwadratowym 
% dolnoprzepustowego filtra FIR o symetrycznej odpowiedzi 
% impulsowej typu I i dlugosci N=2M+1
% z wazona charakterystyka amplitudowa
% Omc - krawedz pasma w radianach
% Wp, Ws - wagi w pasmie przepustowym i zaporowym
%
% Przyklad 4.3.
% m-plik (funkcyjny): fir_mse_W.m
%
% Cyfrowe przetwarzanie sygnalow. Podstawy, multimedia, transmisja. PWN, Warszawa, 2014.
% Rozdzial 4. Projektowanie filtrow. 
% Autorzy rozdzialu: Krzysztof Duda, Przemyslaw Korohoda.
%
% Wiecej na stronie internetowej: http://teledsp.kt.agh.edu.pl
%
% Opracowanie przykladu: kwiecien 2013 r./KD.

dOm= pi/1e3;          %krok calkowania
Om = -pi: dOm : pi-dOm; 
NOm= length(Om);
OmD= ones(size(Om)); %zadana charakterystyka amplitudowa
ind= find(abs(Om)>Omc);
OmD(ind) = 0;
OmW = Wp*ones(size(Om)); %funkcja wazaca
OmW(ind) = Ws*ones(size(ind));
R = zeros(M+1,M+1); 
for n=0:M
    for m=0:M
       R(n+1,m+1)= sum( OmW.*cos(Om*n).*cos(Om*m) )/NOm;
    end    
end
iR = inv(R);
c  = zeros(M+1,1);
for k=0:M
    c(k+1,1) = sum( OmW.*OmD.*cos(k*Om) )/NOm;
end
a = iR*c;
for k=1:M
   h(k) = a(k+1)/2;  
end
h = [fliplr(h) a(1) h];

% KONIEC PRZYKLADU 4.3.