
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%  Przyklad 5.5 CALY 
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%  Testowanie roznych algorytmow szybkiego splotu sygnalow, uzywajacych FFT
%  Autor: Tomasz Zielinski, tzielin@agh.edu.pl
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

clear all; close all; clf;

Nh = 512;       % dlugosc filtra
Nb = Nh;        % dlugosc fragmentu sygnalu dla metody sekcjonowanego splotu overlap-add
Nx = 10*Nh;     % dlugosc calego sygnalu
fp = 44100;     % czestotliwosc probkowania

f1 = 100;       % czestotliwosci trzech sinusoid [kHz]	
f2 = 1000;      %
f3 = 10000;     %

Wn = 500/(fp/2);                    % czestotliwosc graniczna (unormowana) filtra LowPass
dt = 1/fp; t = 0 : dt : dt*(Nx-1);  % czas

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Odpowiedz impulsowa filtra
%h=fir1(Nh-1,Wn);  % filtr FIR, Nh wspolczynnikow
h=0.99.^(0:Nh-1);
plot(h); title('Odpowiedz impulsowa filtra');
pause

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Sygnal
x1=sin(2*pi*f1*t);
x2=sin(2*pi*f2*t);
x3=sin(2*pi*f3*t);
%x=x1+x2+x3;       % suma trzech sinusoid
x=randn(1,Nx);
plot(t,x);title('Sygnal');
pause;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Metoda pierwsza: instrukcja "conv"
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
tic
y1 = conv(x,h);
toc
figure(1)
subplot(6,1,1);
plot(y1); title('#1: Splot bezposredni');
pause;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Metoda druga: szybki splot calego sygnalu
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
tic
xz  = [ x                zeros(1,Nh-1)];
hzz = [ h zeros(1,Nx-Nh) zeros(1,Nh-1)];
y2 = ifft( fft(xz) .* fft( hzz ) );
toc
subplot(6,1,2); plot(y2); title('#2: Szybki splot calego sygnalu');
blad2 = max(abs(y1-y2))
pause

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Metoda trzecia: overlap-save / szybki splot fragmentow sygnalu
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

tic
L = ceil( log2(2*Nh) ); Ns = 2^L;      % Ns - dlugosc segmentu dla metody overlap-save ( >> M ) 
Ls = floor( (Nx-(Nh-1))/(Ns-(Nh-1)) ); % Ls - liczba segmentow (w kazdym wspolnych jest Nh-1 probek)
hz = [ h zeros(1,Ns-Nh) ];             % dodaj zera do h
HZ = fft(hz);                          % FFT z hz
for i = 0 : Ls-1                       % pobierz kolejne bloki probek do bufora 
    n1st = 1+i*(Ns-(Nh-2)-1);          % indeks poczatku kolejnego bloku
    bx = x( n1st : n1st + Ns-1 );      % fragmenty sygnalow w kolejnych, nakladajacych sie blokach
    BX = fft(bx);                      % FFT fragmentu sygnalu
    ys = real( ifft( BX .* HZ) );      % odwrotne FFT z iloczynu widm
    y3(n1st+Nh-1+0 : n1st+Nh-1+(Ns-Nh)) = ys(Nh:Ns); % obetnij niepoprawny poczatek w ys[n], reszte wstaw do y3[n]
end
toc

Ny3 = length(y3);
y3 = [ y3 zeros(1,length(y1)-length(y3)) ];
subplot(6,1,3); plot(y3); title('#3: Splot metoda overlap-save');
n = Nh:Ny3;
blad3 = max(abs(y1(n)-y3(n)))
pause

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Metoda czwarta: overlap-add #1 / szybki splot fragmentow sygnalu
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

tic
Lb = ceil(Nx/Nb);                           % liczba blokow
hzz = [h zeros(1,Nb-Nh) zeros(1,Nh-1)];     % wypelnianie h zerami
Hz = fft(hzz);                              % fft odpowiedzi impulsowej
y4 = zeros(1,Nh-1);                         % przygotowanie wektora y4
for k = 1:Lb
    bx=x((k-1)*Nb+1:k*Nb);       % wycinanie bloku Nb probek sygnalu x[n]
    bxz=[bx zeros(1,Nh-1)];      % uzupelnianie zerami do dlugosci Nb+Nh-1
    Bxz=fft(bxz);
    by=real(ifft(Bxz.*Hz)); 
    y4(end-(Nh-2):end) = y4(end-(Nh-2):end)+ by(1:Nh-1);
    y4 = [ y4 by(Nh:end)];
end;
toc

subplot(6,1,4); plot(y4); title('#4: Splot metoda overlap-add #1');
blad4 = max(abs(y1-y4))
pause;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Metoda piata: overlap-add #2 (inne dodawanie zer) / szybki splot fragmentow sygnalu
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

tic
Lb = ceil(Nx/Nb);                           % liczba blokow
hzz = [h zeros(1,Nb-Nh) zeros(1,Nh)];       % wypelnianie h zerami
Hz = fft(hzz);                              % fft odpowiedzi impulsowej
y5 = zeros(1,Nh-1);                         % przygotowanie wektora y5
for k = 1:Lb
    bx = x((k-1)*Nb+1:k*Nb);           % wycinanie bloku Nb probek sygnalu x[n]
    bxz = [bx zeros(1,Nh)];            % uzupelnianie zerami do dlugosci Nb+Nh-1
    Bxz = fft(bxz);
    by = real(ifft(Bxz.*Hz)); 
    y5(end-(Nh-2):end) = y5(end-(Nh-2):end)+ by(1:Nh-1);
    y5 = [ y5 by(Nh:end-1)];
end;
toc

subplot(6,1,5); plot(y5); title('#5: Splot metoda overlap-add #2');
blad5 = max(abs(y1-y5))
pause;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Metoda szosta: overlap-add #3 / szybki splot fragmentow sygnalu
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

tic
Lb = ceil(Nx/Nb);                           % liczba blokow
hzz = [h zeros(1,Nb-Nh) zeros(1,Nb)];       % wypelnianie h zerami Nb+Nb
Hz = fft(hzz);                              % fft odpowiedzi impulsowej
J = (-1).^(0:2*Nb-1);                       % macierz pomocnicza
X2 = zeros(1,2*Nb);                         % wypelnienie X2 zerami
y6 = [];                                    % przygotowanie wektora y6
for k = 1:Lb
    x1 = x((k-1)*Nb+1:k*Nb);     % wycinanie bloku Nb probek sygnalu x[n]
    xx = [x1 zeros(1,Nb)];       % uzupelnianie zerami do dlugosci Nb+Nh
    X1 = fft(xx);
    X = X1 + J.*X2;
    X2 = X1;
    yy = real(ifft(X.*Hz));
    y6 = [ y6 yy(1:Nb)];
end;
toc

Ny6 = length(y6); n6 = 1 : Ny6;
y6 = [ y6 zeros(1,Nb) ];
subplot(6,1,6); plot(y6); title('#6: Splot metoda overlap-add #3');
blad6 = max(abs(y1(n6)-y6(n6)))
pause;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Metoda siodma: overlap-add z podzialem h #1 / szybki splot fragmentow sygnalu
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

tic
Nb = 32;              % dlugosc bloku probek
Lb = ceil(Nx/Nb);     % liczba blokow sygnalu
Lh = ceil(Nh/Nb);     % liczba blokow odpowiedzi impulsowej

for k = 1:Lh
    H(k,1:2*Nb) = fft( [h((k-1)*Nb+1:k*Nb) zeros(1,Nb)] );   % wypelnianie h zerami
end

y7 = zeros(1,Nh);
for k = 1:Lb
    y7 = [y7 zeros(1,Nb)];
    X = fft( [x((k-1)*Nb+1:k*Nb) zeros(1,Nb)] ); % wycinanie bloku Nb probek sygnalu x[n]
    for l = 1 : Lh
        by = real(ifft(X.*H(l,:)));
        indx = (k-1)*Nb+(l-1)*Nb;
        y7( indx+1:indx+2*Nb) = y7(indx+1:indx+2*Nb) +  by;
    end
end;
toc

figure(2)
subplot(1,1,1); plot(y7); title('#7: Splot metoda overlap-add z podzialem h #1');
blad7 = max(abs(y1-y7(1:end-1)))
pause;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Metoda osma: overlap-add z podzialem h #2 / szybki splot fragmentow sygnalu
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

tic
Nb = 32;              % dlugosc bloku probek
Lb = ceil(Nx/Nb);     % liczba blokow sygnalu
Lh = ceil(Nh/Nb);     % liczba blokow odpowiedzi impulsowej

for k=1:Lh
    H(k,1:2*Nb) = fft( [h((k-1)*Nb+1:k*Nb) zeros(1,Nb)] );   % wypelnianie h zerami
end

bx = zeros(1,2*Nb);
X = zeros(Lh,2*Nb);
y8 = [];
for k = 0 : Lb-1
    bx = [ bx(Nb+1:2*Nb) x(k*Nb+1:k*Nb+Nb) ];      % pobierz kolejny blok probek x[n]
	X = [ fft(bx); X(1:Lh-1,:) ];  % FFT
    XH = X.*H;
    by = real(ifft(sum(XH)));
    y8 = [ y8 by(Nb+1:2*Nb) ];
end
toc

len = min( length(y1), length(y8) );
y1r = y1(Nh+1:len);
y8 = y8(Nh+1:len);
len = length(y8);

figure(3)
subplot(1,1,1); plot(1:len,y1r(1:len),'r',1:len,y8(1:len),'b');
title('#8: Splot metoda overlap-add z podzialem h #2');
blad8 = max(abs(y1r-y8))
pause;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Metoda dziewiata: overlap-save z podzialem h #1 / szybki splot fragmentow sygnalu
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

tic
Nb = 32;            % dlugosc bloku probek
Lb = ceil(Nx/Nb);   % liczba blokow sygnalu
Lh = ceil(Nh/Nb);   % liczba blokow odpowiedzi impulsowej

for k=1:Lh
    H(k,1:2*Nb) = fft( [h((k-1)*Nb+1:k*Nb) zeros(1,Nb)] );   % wypelnianie h zerami
end

y9 = zeros(1,Nh);
for k = 1:Lb-1
    X = fft( [ x((k-1)*Nb+1:(k-1)*Nb+2*Nb) ] );              % wycinanie bloku Nb probek sygnalu x[n]
    for l = 1 : Lh
        by = real(ifft(X.*H(l,:)));
        indx = (k-1)*Nb+(l-1)*Nb;
        y9( indx+1 : indx+Nb ) = y9( indx+1 : indx+Nb ) +  by(Nb+1:2*Nb);
    end
    y9 = [y9 zeros(1,Nb)];
end;
toc

subplot(1,1,1);
nstart = Nh+1;
n1 = nstart+1:nstart+Nx/2;
n9 = n1-Nb;

figure(4); plot(n1,y1(n1),'r',n1,y9(n9),'b');
title('#9: Splot metoda overlap-save z podzialem h');
blad9 = max(abs(y1(n1)-y9(n9)))
pause;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Metoda dziesiata! Podzial odpowiedzi impulsowej filtra na NIEROWNE kawalki
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

tic
N = 32;                % rozmiar pierwszego (najmniejszego) bloku probek
max_index = Nh/(4*N);  % dlugosc najwiekszego bloku (wyrazona w wielokrotnosci N)
H = zeros((log2(max_index)+1)*2, Nh/2);
ile_odci = size(H,1);
y10 = zeros(1,Nx+2*Nh-1);

i=1; k=1;              % indeksy
while i<=ile_odci      % podzial odpowiedzi filtra
   H(k  ,1:i*N) = h(2*N*i+1 : 2*N*i+N*i);   % odcinki filtra wpisywane sa do macierzy
   H(k+1,1:i*N) = h(3*N*i+1 : 3*N*i+N*i);   % o wymiarze: Nh/2 x log2(max_index)
   i=i*2; k=k+2;       % zwiekszenie indeksow
end

   					   % FFT odcinkow filtra   
i=1;                   % inicjacja zmiennej
for k=1:2:size(H,1)    % FFT odcinkow (w jednym przebiegu petli liczymy dwa kolejne)
   H(k,  1:2*i*N) = fft(H(k,   1:2*i*N));    % kazdy odcinek jest wypelniony zerami do 
   H(k+1,1:2*i*N) = fft(H(k+1, 1:2*i*N));    % dlugosci 2*dlugosc odcinka
   i=i*2;
end;

x = [ x zeros(1,Nh-1) ];   % wypelnienie zerami sygnalu - do dlugosci odp. impulsowej

for n = 1 : Nx+Nh-1        % powtarzamy do konca sygnalu
   
   i = 1;
        
   %pierwszy kawalek metoda bezposrednia
   if n<2*N
    % splot = h(Nh-n+1:Nh)*(x(1:n))'; 
      splot = h(n:-1:1)*(x(1:n))';
   else
    % splot = h(Nh-2*N+1:Nh)*(x(n-2*N+1:n))';
      splot = h(2*N:-1:1)*(x(n-2*N+1:n))';
   end;
   y10(n) = y10(n)+splot;

   %dalej metoda blokowa
   k = 1;
   while(i <= max_index)
    % n,i,k,pause 
      if mod(n,i*N)==0                         % czy mamy odp. liczbe probek (32, 64, 128, 256 ...)
         bx=[x(n-i*N+1:n),zeros(1,i*N)];       % wycinanie kawalka sygnalu
         BX = fft(bx);				           % FFT 
         XH1 = BX.*H(k,   1:2*(i*N));          % w jednym przebiegu
         XH2 = BX.*H(k+1, 1:2*(i*N));          % dwa sploty
         i1 = n+i*N;						   % gdzie wstawic wynik?
         i2 = n+2*i*N;                         % to samo dla drugiego odcinka
         Ny = 2*i*N;                           % dlugosc odcinka sygnalu
         y10(i1+1:i1+Ny) = y10(i1+1:i1+Ny) + real(ifft(XH1));
         y10(i2+1:i2+Ny) = y10(i2+1:i2+Ny) + real(ifft(XH2));
         k=k+2;
      else
         break % jezeli ilosc probek nie dzieli sie przez N, to nie dzieli sie tez przez 2*N, 4*N itd.
      end
      i=i*2;
   end
end
toc

y10 = y10(1:length(y10)-Nh);
figure(5); subplot(1,1,1); plot(y10);
title('#10: Overlap-add: podzial h na NIEROWNE kawalki');
blad10 = max(abs(y1-y10))
pause

clear H;