% Przyklad 6.4:
function przyklad_06_04()

rand('state', sum(100 * clock));          % Inicjalizacja generatora liczb losowych.
phi_start = 0;                                 % Ustalenie rozwiazania poczatkowego.
options = optimset('MaxFunEvals', 2000, ... % Parametry algorytmu optymalizacyjnego.
    'DiffMinChange', 0.00001, 'DiffMaxChange', 0.15, 'TolFun', 1e-10, 'TolX', 1e-10);
phi_opt = fminsearch(@obj_fun, phi_start, options);  % Optymalizacja bez ograniczen.
obj_fun(phi_opt);                % Prezentacja najlepszego znalezionego rozwiazania.

%===================================================================================
function err = obj_fun(phi)                            % Funkcja celu optymalizacji.

% Zamiast bezposrednio optymalizowac czestotliwosc ...
% odciecia optymalizujemy jej odchylenie od wartosci nominalnej.
% Dzieki temu nie trzeba uwzgledniac ograniczen podczas optymalizacji.

M = 32;    N = 512;                 % Liczba kanalow i dlugosc filtru prototypowego.
fc = (1 + sin(phi * pi) / 2) / (2 * M);         % Ustalenie czestotliwosci odciecia.
p = fir1(N - 1, fc, kaiser(N,9)); % Synteza prototypu z wykorzystaniem okna Kaisera.
L = 16 * 512;                         % Liczba punktow charakterystyki amplitudowej.
P = abs(fft(p,L));   % Obliczenie charakterystyki amplitudowej filtru prototypowego.
Px = P(1 : (L / (2 * M)));      % Wybranie jej kluczowego fragmentu i obliczenie ...
T0 = Px .^ 2 + fliplr(Px) .^ 2;     % charakterystyki amplitudowej transmitancji ...
% znieksztalcen. Maksymalne odchylenie tej charakterystyki od 1 stanowi miare bledu.
err = max(abs(T0 - 1)); 

if (nargout == 0),    % Jezeli funkcja wywolywana spoza algorytmu optymalizacji, ...
    figure;                                             % to prezentacja bledu i ...
    plot((1 : (L / (2 * M))) / L, T0);                     % charakterystyk filtrow.
    box on;   grid on;   ylabel('|T(e^{j\Omega})|');   xlabel('\Omega/2\pi');
    text(0.01, (1 + min(T0)) / 2.0005, sprintf('f_c=%1.2e\n\\epsilon=%1.2e', ...
        fc, err), 'BackgroundColor', 'white', 'EdgeColor', 'black')
    
    fb_show_prototype(p, 2 *512, 1 / (2 * M));
    fb_show_response(fb_cos_modulated_filters(p, 32), 2 * 512);
end