% Przyklad 6.8
function przyklad_06_08()

M = 8;                                                             % Liczba kanalow.
S = M;                     % Wspolczynnik podprobkowania. Np. probkowanie krytyczne.
p = [ ...                  % Prototyp, ktory zapewnia perfekcyjna rekonstrukcje. 
   -0.010752854659614  -0.010355843731479  -0.009021456700855   0.002051663347521 ...
   -0.005105353272382   0.004546994351895   0.006046243066294   0.007860575558116 ...
    0.011110865371895   0.013752254564423   0.015066834172795   0.006057503008643 ...
    0.015073473357605   0.007593985334780   0.008029232186393   0.008122289339580 ...
    0.003096871476097  -0.000046421944715  -0.004004322195056  -0.006153938847589 ...
   -0.014199319499011  -0.020476035468850  -0.028097809130004  -0.035829516847893 ...
   -0.042625702216762  -0.048277268035077  -0.053288660523956  -0.059563614870500 ...
   -0.061081971560308  -0.064426516985650  -0.065535659975704  -0.065843520373467 ];
% Wystarcza podanie polowy wspolczynnikow, bo odpowiedz impulsowa jest symetryczna.
p = [p fliplr(p)]; 
x = [1 : 9 zeros(1, 127)];                                       % Sygnal wejsciowy.
xs = fb_cos_analysis(p, M, S, x);         % Analiza podpasmowa. Sygnaly kanalowe ...
                                            % w wierszach macierzy wynikowej.
ys = xs; % Brak przetwarzania kanalow, aby sprawdzic czy zespol filtrow zapewnia PR.

y = fb_cos_synthesis(p, S, ys);        % Synteza wyjsciowego sygnalu pelnopasmowego.
fb_show_io(x, y, 512);       % Prezentacja sygnalu oryginalnego i zrekonstruowanego.

%===================================================================================
function xs = fb_cos_analysis(p, M, S, x)         % Polifazowa realizacja czesci ...
                              % analizujacej zespolu filtrow z modulacja kosinusowa.

ss = zeros(2 * M, length(x) / S);      % Sygnaly z wyjsc skladowych polifazowych ...
                                            % (prealokacja pamieci).
s = x;                         % Sygnal z wyjscia lancucha opoznien (inicjalizacja).
pp = reshape(p, 2 * M, length(p) / (2 * M));     % Przygotowanie wspolczynnikow  ...
pp(:, 2 : 2 : end) = -pp(:, 2 : 2 : end);	           % skladowych polifazowych ...
pp = pp(:);                                                  % filtru prototypowego.
for n = 1 : length(pp),       % Iteracja po wspolczynnikach skladowych polifazowych.
    m = mod(n - 1, 2 * M) + 1;            % Ustalenie indeksu skladowej polifazowej.
    ss(m, :) = ss(m, :) + pp(n) ...                 % Dodanie kolejnej skladowej ...
        .* s(1 : S : end);                         %  do wyjscia komponentu.
    s = filter([0 1], 1, s); % Wyznaczenie sygnalu na wyjsciu nastepnego opoznienia.
end
xs = acosmodmtx(M, length(p)) * ss; % Modulacja, ktorej wynikiem sa sygnaly kanalowe

%===================================================================================
function y = fb_cos_synthesis(p, S, ys)           % Polifazowa realizacja czesci ...
% syntezujacej zespolu filtrow rownomiernych z modulacja kosinusowa.

M = size(ys, 1); % Liczba kanalow okreslona rozmiarem macierzy sygnalow podpasmowych
pp = reshape(p, 2 * M, length(p) / 2 / M); % Ustalenie wspolczynnikow skladowych ...
pp(:, 2 : 2 : end) = -pp(:, 2 : 2 : end);       % polifazowych filtru prototypowego.
pp = -pp(:) * S;           % Polaczenie filtracji ze wzmocnieniem o wspolczynnik ...
               % podprobkowania $S$ w celu poprawnego odtworzenia amplitudy sygnalu.

ts = scosmodmtx(M, length(p))' * ys;   % Demodulacja, ktorej wynikiem sa sygnaly ...
                                   % wchodzace do skladowych polifazowych prototypu.

y = zeros(1, S * size(ys, 2));   % Sygnal wyjsciowy lancucha opoznien (prealokacja).
for n = 1 : length(pp),       % Iteracja po wspolczynnikach skladowych polifazowych.
    y = filter([0 1],1,y); % Wyznaczenie sygnalu na wyjsciu odpowiedniego opoznienia
    m = mod(n - 1, 2 * M) + 1; %            Ustalenie indeksu skladowej polifazowej.
    y(1 : S : end) = ...                    % Wyznaczenie sygnalu, ktory wchodzi ...
        y(1 : S : end) + pp(n) * ts(m, :);               % do nastepnego opoznienia.
end                                   % W ostatniej iteracji sygnal zrekonstruowany.

%===================================================================================
function CA = acosmodmtx(M, N)              % Macierz modulacji kosinusowej do  ...
                                            % M-kanalowego zespolu filtrow analizy.

[k, n] = ndgrid(0 : (M - 1), 0 : (2 * M - 1));  % Generacja indeksow i elementow ...
CA = 2 * cos(pi/M .* (k + 0.5) .* (n - N/2 + 0.5) + pi/4 * (-1) .^ k);   % macierzy.

%===================================================================================
function CS = scosmodmtx(M, N)              % Macierz modulacji kosinusowej do  ...
                                            % M-kanalowego zespolu filtrow syntezy.

[k, n] = ndgrid(0 : (M - 1), 0 : (2 * M - 1));              % Generacja indeksow ...
CS = 2 * cos(pi/M .* (k + 0.5) .* ...                       % i elementow macierzy.
    (2 * M - 1 - n - N/2 + 0.5) - pi/4 * (-1) .^ k);