% Przyklad 6.10
function przyklad_06_10()

M = 8;  K = 1;  % Liczba kanalow zespolu filtrow i liczba stopni struktury kratowej.
rand('state', sum(100 * clock));          % Inicjalizacja generatora liczb losowych.
a_start = rand(1, K * M * 2) .* (2 * pi);      % Losowanie rozwiazania poczatkowego.

% Wielokrotne wywolanie algorytmu optymalizacyjnego z roznymi rozwiazaniami ...
% poczatkowymi zwieksza szanse na znalezienie globalnego optimum.

options = optimset('MaxFunEvals', 2000, 'DiffMinChange', 0.0001, 'DiffMaxChange',...
    0.5, 'TolFun', 1e-9, 'TolX', 1e-9);     % Parametry algorytmu optymalizacyjnego.
a_opt = fminunc(@(a)obj_fun(a,M), a_start, options); % Optymalizacja bez ograniczen.
obj_fun(a_opt, M);               % Prezentacja najlepszego znalezionego rozwiazania.

%===================================================================================
function err = obj_fun(a, M)                           % Funkcja celu optymalizacji.

K = length(a) * 2 / M; N = K * M * 2;         % Rzad struktury kratowej. Dlugosc ...
Mx2 = 2 * M;                 % filtru prototypowego. Liczba skladowych polifazowych.
p = zeros(1, N);                          % Macierz wspolczynnikow filtru prototypu.

for k = 1 : (M/2), % Iteracja po parach skladowych polifazowych filtru prototypowego
    [gk, gkplusm] = lattice_tf(a((k-1) * K + [1:K])); % Wyznaczenie wspolczynnikow ...
    p(k : Mx2 : end) = gk;           % pary skladowych i filtru. Poniewaz prototyp ...
    p((N - k + 1) : -Mx2 : 1) = gk;     % ma liniowa charakterystyke fazowa, czyli ...
    p((M + k) : Mx2 : end) = gkplusm;     % symetryczna odp. impulsowa, to te same ...
    p((N - M - k + 1) : -Mx2 : 1) = gkplusm;  % wspolczynniki ma druga para skladowych
end;    p = p ./ (8 * sqrt(2));  % Normalizacja wspolczynnikow filtru prototypowego.

% Ocena prototypu: na ile jego charakterystyka amplitudowa ...
% w pasmie przepustowym odbiega od 1.

L = 512;                              % Liczba punktow charakterystyki amplitudowej.
P = fft(p, L);       % Obliczenie charakterystyki amplitudowej filtru prototypowego.
Pref = zeros(1, L/2);  Pref(1 : (L / (2 *  2 * M))) = 1; % Charakterystyka wzorcowa.
weight = sin((1 : (L / (2 * M))) *  pi / 2 / (L / (1 * 2 * M)));       % Wagi bledu.
weight = [fliplr(weight) 0.75 * weight 0.75 * ones(1, L / 2 - 2 * length(weight))];
err = sum((weight .* (abs(P(1 : L/2)) - Pref)).^ 2); % Miara jakosci charakterystyki

if (nargout == 0), % Jezeli funkcja wywolywana spoza algorytmu optymalizacji, to ...
    figure;   plot(weight);     % prezentacja charakterystyk filtrow i sprawdzenie ...
    fb_show_prototype(p, L, 0.5 / (2 * M));              % perfekcyjnej rekonstrukcji.

    [h,f] = fb_cos_modulated_filters(p, M); fb_show_response(h, L); % Synteza filtrow.
    x  = zeros(1, 1 * 128); x(1 : 5) = 1 : 5;                      % Sygnal wejsciowy.
    xs = fb_analysis(h, M, x);                              % Dekompozycja podpasmowa.
    ys = xs;        % Brak przetwarzania, aby sprawdzic czy perfekcyjna rekonstrukcja.
    y = fb_synthesis(f * M, M, ys);                 % Odtworzenie sygnalu wejsciowego.
    fb_show_io(x, y, 512);
end

%===================================================================================
function [gk, gkplusm] = lattice_tf(alpha)            % Generacja wspolczynnikow ...
                % transmitancji skladowych polifazowych z uzyciem struktury kratowej.

K = length(alpha);
gk = zeros(1, K);           gkplusm = zeros(1, K);           % Prealokacja wektorow.
gk(1) = cos(alpha(1));      gkplusm(1) = sin(alpha(1));        % Stopien podstawowy.
for ii = 2 : K,                     % Iteracja po kolejnych stopniach wydluzajacych.
    c = cos(alpha(ii));     s = sin(alpha(ii));
    gk(ii) = s * gkplusm(ii - 1);   gkplusm(ii) = -c * gkplusm(ii - 1);
    for jj = (ii - 1) : -1 : 2,
        gkplusm(jj) = s * gk(jj) - c * gkplusm(jj - 1);     % Opoznienie polaczone ...
        gk(jj) = c * gk(jj) + s * gkplusm(jj - 1);                        % z obrotem.
    end;
    gkplusm(1) = s * gk(1);     gk(1) = c * gk(1);
end;