
% --------------------------------------------------------------------
% TEST ADAPT
% PROGRAM DO TESTOWANIA FILTROW ADAPTACYJNYCH Z ROZDZIALU 7
% Autor: Tomasz Zielinski, tzielin@ agh.edu.pl
% --------------------------------------------------------------------

% WEJSCIE:  x  = sygnal filtrowany, d  = sygnal odniesienia
% WYSCIE:  	y  = przefiltrowany adaptacyjnie sygnal x, e  = d-y , czyli sygnal bledu
  
  clear all; close all;

% Uruchom program dla wszystkich kombinacji parametrow  ialg  i  itest
  ialg = 4 ;      % rodzaj algorytmu adaptacji (tak jak w rozdziale)
                  % 1=LMS        4=dekor LMS    7=WRLS1
                  % 2=NLMS       5=dekor NLMS   8=WRLS2 (0-start od 0, 1-start z macierza)    
                  % 3=AP-NLMS    6=LMS-Newton 
  
  itest = 1;      %  test: 1 - usuwanie addytywnej interferencji (przydzwieku sieci)
  				  %        2 - usuwanie addytywnego szumu (za pomoca liniowej predykcji) 
  
  M = 50;         % liczba wspolczynnikow (wag) filtra
  nr = [ 1 ];     % historia zmiennosci wag h[k], numery sledzonych wag
  ifig = 0;       % rysynki: 0/1, czyli NIE/TAK
  
% Generacja sygnalu testowego LFM (liniowa modulacja czestotliwosci)
  Nx = 1000;      %  liczba probek
  fpr = 1000;     %  czestotliwosc probkowania
  A = 1;          %  amplituda
  f0 = 0;         %  czestotliwosc poczatkowa sygnalu
  df = 25;        %  przyrost czestotliwosci Hz/sek
  obw  = 1;       %  wybor obwiedni czasowej: 0=brak, 1=Gauss, 2=alfa*t, 3=exp(-alfa*t)
  dt=1/fpr; t=0:dt:(Nx-1)*dt;                             % chwile czasowe probkowania
  s = A*cos( 2*pi* (f0*t + 0.5*df*t.^2) );                % sygnal LFM
  if (obw == 1) alfa=10; w=exp(-alfa*pi*(t-0.5).^2); end  % obwiednia 1 - krzywa Gaussa
  if (obw == 2) alfa=5; w=alfa*t; end                     % obwiednia 2 - alfa*t
  if (obw == 3) alfa=5; w=exp(-alfa*t); end               % obwiednia 3 - exp(-alfa*t)
  if (obw~=0) s = s .* w; end                             % sygnal z obwiednia
  if (itest==1)                     %  TEST 1 - kasowanie interferencji
     P = 0;                         %  brak predykcji
     x = 0.1*sin(2*pi*50*t-pi/5);   %  siec przesunieta w fazie
     d = s + 0.5*sin(2*pi*50*t);    %  sygnal + siec
  end
  if (itest==2)                     %  TEST 2 - odszumianie droga predykcji
     P = 1;                         %  rzad predykcji (do zmiany: 1,2,3,...)
     x = s + 0.25*randn(1,Nx);      %  sygnal + szum
     d = [ x(1+P:length(x)) 0 ];    %  odniesieniem sygnal "przyspieszony" o P probek
  end
  
% Rysunki sygnalow wejsciowych  
  subplot(211); plot(t,x); grid; title('WE : sygnal x');
  subplot(212); plot(t,d); grid; title('WE : sygnal d');
  xlabel('czas [sek]'); pause

% Parametry adaptacji  
  mi=0; gamma=0; lambda=0; delta=0; alfa=0; L=0; leaky=0;
  if(ialg==1)  mi=0.1; end                                           % LMS 0.1
  if(ialg==2)  mi=0.025; gamma=0.001; end                            % NLMS
  if(ialg==3)  L=4; mi=0.05; gamma=0.001; end                        % AP-LMS
  if(ialg==4)  mi=5; end                                             % zdekorelowany LMS
  if(ialg==5)  mi=1; gamma=0.001; end                                % zdekorelowany NLMS
  if(ialg==6)  mi=0.5; lambda=0.9999; alfa=1-lambda; delta=0.1; end  % LMS-Newton
  if(ialg==7)  lambda=0.9999; delta=0.5; end                         % WRLS1
  if(ialg==8)  lambda=0.9999; delta=0.5; end                         % WRLS2

  % Filtracja adaptacyjna  
  if(ialg<=6)
      [y1, e1, h1, hest ] = filtrNLMS_rys(M,L,d,x,nr,mi,gamma,alfa,delta,leaky,ialg,ifig);
      [y2, e2           ] =     filtrNLMS(M,L,d,x,   mi,gamma,alfa,delta,leaky,ialg     );
      max(abs(y1-y2)), max(abs(e1-e2)), pause
  end
  if(ialg==7)
      [y1, e1, h1, hest ] = filtrWRLS1_rys(M,d,x,nr,lambda,delta,ifig);
      [y2, e2           ] =     filtrWRLS1(M,d,x,   lambda,delta     );
      max(abs(y1-y2)), max(abs(e1-e2)), pause
  end
  if(ialg==8)
      [y1, e1, h1, hest ] = filtrWRLS2_rys(M,d,x,nr,lambda,delta,1,ifig); % 1=start macierz
      [y2, e2           ] =     filtrWRLS2(M,d,x,   lambda,delta,1     );    % 1=start macierz
      max(abs(y1-y2)), max(abs(e1-e2)), pause
  end

% Rysunki sygnalow wyjsciowych
  y = y2; e = e2;
  subplot(211); plot(t,y); grid; title('WY : sygnal y = dest');
  subplot(212); plot(t,e); grid; title('WY : sygnal e = err');
  xlabel('czas [sek]'); pause
  if (itest==1)
     subplot(111); plot(t,s,'r',t,e,'b'); grid; xlabel('czas [sek]');
     title('Orginal (czerwony) i wynik filtracji (niebieski)'); pause
  end
  if (itest==2)
     n=1:Nx-P;
     subplot(111); plot(t(n),s(n),'r',t(n),y(n),'b'); grid; xlabel('czas [sek]');
     title('Orginal (czerwony) i wynik filtracji (niebieski)'); pause
  end
