function [y,e,gest,gamma] = filtrGLWF_rys(M,d,x,nr,lambda,beta)
% Adaptacyjny gradientowy filtr kratowy Wienera typu FIR (Gradient Lattice Wiener Filter)
% Rys. 8.7b tylko z adaptacja wag filtra
% Na podstawie [Orfa88]
 
%w = zeros(1,M+1);           % stany wewnetrzne kazdej sekcji
w = 0.000001*randn(1,M+1);

g = w;   gamma = w;   da = w;   db = w;
init = 0;

gest = [];

for n = 1 : length(x)        % dla kolejnych probek sygnalu x[n]
    ea(1) = x(n);          % wejscie do pierwszej sekcji filtra
    eb(1) = x(n);          % wejscie do pierwszej sekcji filtra
    y(n) = g(1)*eb(1);     % estymata y[n] bazujaca na sekcji 1
    e(n) = d(n) - y(n);    % estymata e[n] bazujaca na sekcji 1

    db(1) = lambda*db(1) + eb(1)*eb(1);      % uaktualnienie db(1), rownanie (8.116)
    g(1) = g(1) + beta*e(n)*eb(1)/db(1);     % uaktualnienie g(1), rownanie (8.115)
    for k = 2:M+1                            % kolejne sekcje filtra
        ea(k) = ea(k-1) - gamma(k)*w(k);     % rownanie (8.2), czyli ,,motylek"
        eb(k) = - gamma(k)*ea(k-1) + w(k) ;  % 

        da(k) = lambda*da(k) + ea(k-1)*ea(k-1) + w(k)*w(k);            % nowe d[k], (8.95)(8.96)
        gamma(k) = gamma(k) + beta*(ea(k)*w(k)+eb(k)*ea(k-1))/da(k);   % nowe gamma[k], (8.97)

        y(n) = y(n) + g(k)*eb(k);          % estymata x[n] rzedu "k", rown. (8.84)
        e(n) = e(n) - g(k)*eb(k);          % blad e[n] estymaty rzedu "k", rownanie (8.86)

        db(k) = lambda*db(k)+eb(k)*eb(k);  % uaktualnienie dk?[k] (8.116)
        if(k<=init)
            g(k) = g(k) + beta*e(n)*eb(k)/db(k); % uaktualnienie g[k] (8.115)
        end
        w(k) = eb(k-1);                     % aktualizacja stanu wewnetrznego k-tej sekcji filtra kratowego)
   end
   init = init+1;
   gest = [ gest; gamma( nr+1 ) ];
end

