function [y,e,gest,gamma] = filtrWRLS_lattice(M,d,x,nr,lambda)
% Double/Direct WRLS Lattice
% Based on [Orfa88]

%w0 = zeros(1,M+1);
w0 = 0.000001*randn(1,M+1);

w1 = w0; gamma = w0; gammb = w0; D1a = w0; D=w0; g=w0;
init = 0;

gest = [];

for n = 1 : length(x)
    e0a(1) = x(n);        % wejscie do pierwszej sekcji filtra a priori
    e0b(1) = x(n);        % j.w.
    e1a(1) = x(n);        % wejscie do pierwszej sekcji filtra a posteriori
    e1b(1) = x(n);        % j.w.
    
    D1a(1) = lambda*D1a(1) + e1a(1)*e0a(1);   % uaktualnienie D1a(1)
    D1b(1) = lambda*D(1) + e1b(1)*e0b(1);     % uaktualnienie D1b(1)
   
    e0(n) = d(n) - g(1)*e0b(1);              % blad a priori rzedu 0
    g(1) = g(1) + e0(n)*e1b(1)/D1b(1);       % uaktualnienie g(1)
    y(n) = g(1)*e1b(1);               % estymata y[n] bazujaca na sekcji 1
    e(n) = d(n) - y(n);               % estymata e[n] bazujaca na sekcji 1
    
    for k = 2:M+1
        e0a(k) = e0a(k-1) - gammb(k)*w0(k);    % rekursja a posteriori
        e0b(k) = w0(k) - gamma(k)*e0a(k-1);
        
        if(k<=init)                            % uaktualnienie wsp. odbicia
            gamma(k) = gamma(k) + e0b(k)*e1a(k-1)/D1a(k-1); 
            gammb(k) = gammb(k) + e0a(k)*w1(k)/D(k-1); 
        end
        
        e1a(k) = e1a(k-1)- gammb(k)*w1(k);      % rekursja a posteriori
        e1b(k) = w1(k)- gamma(k)*e1a(k-1);
        
        D1a(k) = lambda*D1a(k) + e1a(k)*e0a(k);
        D1b(k) = lambda*D(k) + e1b(k)*e0b(k);
        
        w0(k) = e0b(k-1);           % uaktualninie stanu a priori
        w1(k) = e1b(k-1);           % uaktualninie stanu a posteriori
        D(k) = D1b(k);              % D jest opozniona wersja D1b
        
        e0(n) = e0(n) - g(k)*e0b(k);      % blad a priori rzedu "p"
        if(k<=init)
           g(k) = g(k) + e0(n)*e1b(k)/D1b(k); % uaktualnienie g(k)
        end
        
        e(n) = e(n) - g(k)*e1b(k);  % blad a posteriori rzedu "p"
        y(n) = y(n) + g(k)*e1b(k);  % estymata a posteriori x
        
    end
    D(1) = D1b(1);
    init = init+1;
    
    gest = [ gest; gamma( nr+1 ) ];
end    
