
% --------------------------------------------------------------------------------
% Przyklad 8.4 + 8.5 TEST
% Program do testowania kratowego filtra liniowej predykcji z przykladow 8.4 i 8.5
% Autor: Tomasz Zielinski, tzielin@agh.edu.pl
% --------------------------------------------------------------------------------

% WEJSCIE:  x = sygnal filtrowany,
%           d  = sygnal odniesienia
% WYSCIE:  	y = przefiltrowany adaptacyjnie sygnal x,
%           e = d-y, czyli sygnal bledu
%           h = dwie wybrane wagi filtra

  clear all; close all;

  itest = 2;      % 1 lub 2 skladnikowy sygnal
  ialg = 1;       % 1 lub 2 standardowy LMS lub NLMS jako odniesienie
  ifig = 0;       % rysunki w NLMS 0/1
  M = 25;         % liczba wspolczynnikow (wag) filtra
  P = 1;          % opoznienie sygnalu x[n] w stosunku do d[n] (do zmiany: 1,2,3,...)
  nr = 1:4;       % numery zwracanych, przykladowych wag filtra
  asz = 0.10;     % odchylenie standardowe szumu
 
% Generacja sygnalu testowego - jedna lub dwie sinusoidy
 
  Nx = 1000;                                       % liczba probek
  fpr = 1000;                                      % czestotliwosc probkowania
  dt=1/fpr; t=0:dt:(Nx-1)*dt;                      % chwile probkowania
  s1 = 1*cos( 2*pi*67.89*t );                            % sygnal 1
  s2 = 1*cos( 2*pi*67.89*t ) + 2*cos( 2*pi*123.45*t );   % sygnal 2
  if(itest==1)
     s = s1;                                       % wybor sygnalu 1 lub 2
     K = 1;                                        % liczba skladowych: 1 lub 2
  else
     s = s2;                                       % wybor sygnalu 1 lub 2
     K = 2;                                        % liczba skladowych: 1 lub 2
  end    
  
  x = s + asz*randn(1,Nx);             % sygnal + szum
  d = [ x(1+P:length(x)) zeros(1,P) ]; % odniesienie to sygnal "przyspieszony" o P probek
  
  Nx = Nx-P; x = x(1:Nx); d = d(1:Nx); t=0:dt:(Nx-1)*dt;
  
%  Ewentualne dodatkowe zaszumienie 
   x = x + 0.001*randn(1,Nx);
   d = d + 0.001*randn(1,Nx);

% Rysunki sygnalow wejsciowych  
  subplot(211); plot(t,x); grid; title('WE : sygnal x');
  subplot(212); plot(t,d); grid; title('WE : sygnal d');
  xlabel('czas [sek]'); pause
  
% Wspolczynniki optymalnego filtra predykcji liniowej
  rxx = corrxy(x,x,M+1);
  Rxx = toeplitz(rxx(1:M),rxx(1:M));  % symetryczna macierz autokorelacji x[n]
  a_opt = - Rxx\rxx(2:M+1)';          % wagi optymalnego filtra predykcji
  a_opt = [1 a_opt']
  figure(1)
  subplot(211); stem(a_opt); title('a[n] opt'); grid;
  subplot(212); [A, w ] = freqz(a_opt,1,100); stem( w/pi*fpr/2, abs(1./A)); title('A(f) opt'); grid; pause
       
% Standardowy filtr LMS
  leaky=0; gamma=0.001; lambda=0; delta=0; alfa=0; L=0;
  mi = 0.001; 
  [y1, e1, ahist, a] = filtrNLMS(M,L,d,x,nr,mi,gamma,alfa,delta,leaky,ialg,ifig);
  a_std = [ 1 -a ]
  figure(2)
  subplot(211); stem(a_std); title('a[n] std'); grid;
  subplot(212); [A, w ] = freqz(a_std,1,100); stem( w/pi*fpr/2, abs(1./A)); title('A(f) std'); grid; pause

% Filtr kratowy
  lambda=0.999; alfa=0.005; beta=alfa/(1-lambda);
  [ e2, ghist, gamma ] = filtrGLLPF_rys(M,x,nr,lambda,beta);

% Odtworz koncowe wspolczynniki {a} na podstawie koncowego {gamma} - tabela 8.1
  aa = zeros(M,M);
  aa(1,1) = -gamma(1);
  for i=2:M  
      aa(i,i) = -gamma(i);
      for j=1:i-1
          aa(i,j) = aa(i-1,j) - gamma(i)*aa(i-1,i-j);
      end
  end
  a_krata = [ 1 aa(M,:) ]
  figure(3)
  subplot(311); stem(gamma); title('gamma[n]'); grid;
  subplot(312); stem(a_krata); title('a[n] krata'); grid;
  subplot(313); [A, w ] = freqz(a_opt,1,100); stem( w/pi*fpr/2, abs(1./A)); title('A(f) krata'); grid; pause
  
% ##############################
% Rysunki sygnalow wyjsciowych
% ##############################
  ty=dt*(0:length(y1)-1); te=ty;

  figure(4)
  subplot(211); plot(te,e1); grid; title('ADAPT - WY : sygnal e[n] standard');
  subplot(212); plot(te,e2); grid; title('ADAPT - WY : sygnal e[n] krata');
  xlabel('czas [sek]'); pause
  
  figure(5)
  subplot(411); plot(ty,ahist(:,nr(1))); grid; title('ADAPT - WY : a1[n]');
  subplot(412); plot(ty,ahist(:,nr(2))); grid; title('ADAPT - WY : a2[n]');
  subplot(413); plot(ty,ahist(:,nr(3))); grid; title('ADAPT - WY : a3[n]');
  subplot(414); plot(ty,ahist(:,nr(4))); grid; title('ADAPT - WY : a4[n]'); pause

  figure(6)
  subplot(411); plot(ty,ghist(:,nr(1))); grid; title('ADAPT - WY : gamma1[n]');
  subplot(412); plot(ty,ghist(:,nr(2))); grid; title('ADAPT - WY : gamma2[n]');
  subplot(413); plot(ty,ghist(:,nr(3))); grid; title('ADAPT - WY : gamma3[n]');
  subplot(414); plot(ty,ghist(:,nr(4))); grid; title('ADAPT - WY : gamma4[n]'); pause
  
% Estymacja czestotliwosci sygnalow skladowych 

  r = roots( a_opt );
  figure(7); subplot(111); fi = 0:0.1:2*pi; plot(real(r),imag(r),'o', cos(fi), sin(fi), 'b-' );
  title('Miejsca zerowe wielomianu dla OPTIM');
  [dummy indx] = sort( abs(r), 'descend' );
  r = r(indx);
  r = imag( log(r) );
  r = r(find(r>0));
  freq_optim = r(1:K)/(2*pi)*fpr, pause
  
  r = roots( a_std );
  figure(8); subplot(111); fi = 0:0.1:2*pi; plot(real(r),imag(r),'o', cos(fi), sin(fi), 'b-' );
  title('Miejsca zerowe wielomianu dla STANDARD');
  [dummy indx] = sort( abs(r), 'descend' );
  r = r(indx);
  r = imag( log(r) );
  r = r(find(r>0));
  freq_standard = r(1:K)/(2*pi)*fpr, pause
  
  r = roots( a_krata );
  figure(9); subplot(111); fi = 0:0.1:2*pi; plot(real(r),imag(r),'o', cos(fi), sin(fi), 'b-' );
  title('Miejsca zerowe wielomianu dla KRATA');
  [dummy indx] = sort( abs(r), 'descend' );
  r = r(indx);
  r = imag( log(r) );
  r = r(find(r>0));
  freq_krata = r(1:K)/(2*pi)*fpr, pause
  