% --------------------------------------------------------------------------------
% Przyklad 8.6 + 8.7a + 8.7b
%
% Program do testowania filtrow adaptacyjnych: filtrGLWF (8.6) oraz FDAF-FFT (8.7)
% oraz ich POROWNANIA Z INNYMI FILTRAMI, w tym z filtrami opisanymi w rozdziale 7
%
% TEST WSZYSTKICH ZAIMPLEMENTOWANYCH FILTROW ADAPTACYJNYCH
% W CZTERACH SCENARIUSZACH PRACY
%
% Autor: Tomasz Zielinski, tzielin@agh.edu.pl
% --------------------------------------------------------------------------------

% WEJSCIE:  x = sygnal filtrowany,
%           d  = sygnal odniesienia
% WYSCIE:  	y = przefiltrowany adaptacyjnie sygnal x,
%           e = d-y, czyli sygnal bledu
%           h = dwie wybrane wagi filtra

  clear all; close all;

% Uruchom program dla wszystkich kombinacji parametrow  ialg  i  itest
  ialg = 11;  % rodzaj algorytmu adaptacji (tak jak w rozdziale)
              % 1=LMS      4=dekor LMS    7=WLS   8=WRLS_standard   11=GLWF  12=FDAF_over_add
              % 2=NLMS     5=dekor NLMS           9=WRLS_faest               13=FDAF_over_save
              % 3=AP-NLMS  6=LMS-Newton          10=WRLS_lattice             14=FDAF_over_save_part
                  
  itest = 1;      % test: 1 - usuwanie addytywnej interferencji (sieci)
  				  %       2 - usuwanie addytywnego szumu (za pomoca liniowej predykcji) 
                  %       3 - adaptacyjna identyfikacja obiektu (kanalu)
                  %       4 - tlumienie echa
  ifig = 0;       % rysunki w trakcie adaptacji: 1=tak, 0=nie
  istandard = 0;  % czy chcemy standardowy filtr Wienera (nieadaptacyjnie)? 0=nie, 1=tak 
  ikrata = 0;     % czy chcemy kratowy filtr Wienera (nieadaptacyjnie)? 0=nie, 1=tak 

  M = 50;         % liczba wspolczynnikow (wag) filtra
  nr =[ 1, 2 ];   % numery zwracanych, przykladowych dwoch wag filtra
 
% Generacja sygnalu testowego LFM (liniowa modulacja czestotliwosci) z rozna obwiednia czasowa
 
  obw  = 1;       %  wybor obwiedni czasowej: 0=brak, 1=Gauss, 2=alfa*t, 3=exp(-alfa*t)

  Nx = 2048;      %  liczba probek
  fpr = 1000;     %  czestotliwosc probkowania
  A = 1;          %  amplituda
  f0 = 0;         %  czestotliwosc poczatkowa sygnalu
  df = 25;        %  liniowy przyrost czestotliwosci Hz/sek
  dt=1/fpr; t=0:dt:(Nx-1)*dt;                                   % chwile probkowania
  s = A*cos( 2*pi* (f0*t + 0.5*df*t.^2) );                      % sygnal LFM
  
  if (obw == 1) beta=10; w=exp(-beta*pi*(t-t(Nx/2)).^2); end    % obwiednia 1 - krzywa Gaussa
  if (obw == 2) beta=5; w=beta*t; end                           % obwiednia 2 - beta*t
  if (obw == 3) beta=5; w=exp(-beta*t); end                     % obwiednia 3 - exp(-beta*t)
  if (obw~=0) s = s .* w; end                                   % sygnal z obwiednia
  
  if (itest==1)                     %  TEST 1 - kasowanie interferencji
     P = 0;                         %  brak opoznienia sygnalu x[n] w stosunku do d[n] 
     x = 0.1*sin(2*pi*50*t-pi/5);   %  siec przesunieta w fazie
     d = s + 0.5*sin(2*pi*50*t);    %  sygnal + zaklocenie sinusoidalne (siec)
  end
  if (itest==2)                     %  TEST 2 - odszumianie droga liniowej predykcji
     P = 1;                         %  % opoznienie sygnalu x[n] w stosunku do d[n] (do zmiany: 1,2,3,...)
     x = s + 0.25*randn(1,Nx);      %  sygnal + szum
     d = [ x(1+P:length(x)) 0 ];    %  odniesieniem jest sygnal "przyspieszony" o P probek
  end
  if (itest==3)                     %  TEST 3 - adaptacyjna identyfikacja
     P = 0;                         %  brak opoznienia sygnalu x[n] w stosunku do d[n] 
     x = randn(1,Nx);               %  szum gaussowski
     hk = [ -0.987, 2.345 ];        %  odp. impulsowa kanalu, zmieniajaca sie +/-
     M = length( hk );              %  liczba wag
     bx = zeros(1,M);               %  bufor na probki filtrowanego sygnalu x[n]
     for n = 1: Nx
         bx = [ x(n) bx(1:M-1) ];                  % nowa probka x[n] do bufora
         if ( rem( n, 500 ) == 0 ) hk = -hk; end   % cykliczna zmiana odp. impuls. kanalu
         d(n) = sum( bx .* hk );                   % filtracja x[n] przez hk[n]
     end    
  end
  if (itest==4)                                     %  TEST 4 - adaptacyjne tlumienie echa
     [ mB, fpr ] = wavread('mowcaB1.wav');          %  mowca B1 lub B2
     [ mA, fpr ] = wavread('mowcaA.wav');           %  mowca A
     [ eA, fpr ] = wavread('echoA.wav');            %  echo mowcy A (duze i bolesne)
     [ h, fpr  ] = wavread('odp_impuls_11kHz.wav'); % odp impuls pomieszczenia
     mA = mA'; mB = mB'; eA = eA';                  %  wektor pionowy na poziomy
     
     if(1) % Tworzymy swoje wlasne echo mowcy A, na zyczenie
         % h = [ zeros(1,500) 1 zeros(1,500) 0.5 zeros(1,500) 0.25 ];
           h = [ zeros(1,5) 1 zeros(1,10) 0.5 zeros(1,10) 0.25 ];
           stem(h); title('Symulowane h[n] pomieszczenia'); pause
           eA = conv( mA, h );
           if(0)
              soundsc( mA, fpr ),
              soundsc( eA, fpr ),
           end   
     end
     if(0) % Tworzymy swoje wlasne echo mowcy A, wykorzystujac wczytana odpowiedz impulsowa duzego pomieszczenia
         stem(h); title('h[n] duzego pomieszczenia'); pause  
         eA = conv( mA, h );
           if(0)
              soundsc( mA, fpr ),
              soundsc( eA, fpr ),
           end   
     end
       
     Nx = min( [length(mA), length(mB), length(eA)] ); n=1:Nx;  % przyinamy dlugosc sygnalow
     mA = mA(n); mB = mB(n); eA = eA(n);
     
     d = mB + 0.2*mA;                        %  sygnal z mikrofonu: mowca B + echo mowcy A
     x = mA;                                 %  sygnal z glosnika
     Nx = length( d );                       %  liczba probek
     dt=1/fpr; t=0:dt:(Nx-1)*dt; 
  end

%  Ewentualne dodatkowe zaszumienie 
%  x = x + 0.001*randn(1,Nx);
%  d = d + 0.001*randn(1,Nx);
  
% Rysunki sygnalow wejsciowych
  figure
  subplot(211); plot(t,x); grid; title('WE : sygnal x');
  subplot(212); plot(t,d); grid; title('WE : sygnal d');
  xlabel('czas [sek]'); pause

% Parametry dla optymalnej filtracji Wienera

  rxx = corrxy(x,x,M);
  rdx = corrxy(d,x,M);            % ALTERNATYWA obliczenia tylko h[n]
% Rxx = toeplitz(rxx,rxx);        % symetryczna macierz autokorelacji x[n]
% h = Rxx\rdx;                    % wagi optymalnego filtra Wienera
  [L,E,g,h] = firw(M,rdx,rxx);
  gamma = -L(:,1);  % wsp. odbicia
  figure; subplot(111); stem( h ); grid; title('Optymalna odp impulsowa h filtra Wienera'); pause

  lamb = eig( toeplitz(rxx,rxx) );
  lamb = sort(lamb,'descend');
  figure; subplot(111); stem( lamb ); grid; title('Wartosci wlasne macierz Rxx'); pause
  disp('Sugestia wyboru mi')
  mi1_ryzyk = 1/lamb(1),
  mi2_bezp  = 1/sum(lamb), pause

if(istandard==1)  % Czy chcemy filtrowac WIENER STANDARDOWO NIEadaptacyjnie: 0/1
    
% ##################  
% WIENER KLASYCZNIE
% ################## 
  if(1) % Matlab
    y = filter(h,1,x);
    e = d - y;
  else  % My sami
    [y,e] = filtrDWF(M,h,d,x);
  end  
  ty=dt*(0:length(y)-1); te=ty;
  
% Rysunki sygnalow wyjsciowych dla Winera nieadaptaczjnego  
  figure 
  subplot(211); plot(ty,y); grid; title('NON ADAPT - WY : sygnal y = dest');
  subplot(212); plot(te,e); grid; title('NON ADAPT - WY : sygnal e = err');
  xlabel('czas [sek]'); pause
  figure
  if (itest==1)
     subplot(111); plot(t,s,'b',te,e,'r'); grid; xlabel('czas [sek]');
     title('NON ADAPT: Orginal (czerwony) i wynik filtracji (niebieski)'); pause
     n = 500:Nx;
     SNR_we_dB = 10*log10( sum( s(n).^2 ) / sum( (d(n)-s(n)).^2 ) ),
     SNR_wy_dB = 10*log10( sum( s(n).^2 ) / sum( (s(n)-e(n)).^2 ) ), pause
  end
  if (itest==2)
     n=1:Nx-P;
     subplot(111); plot(t(n),s(n),'r',ty(n),y(n),'b'); grid; xlabel('czas [sek]');
     title('NON ADAPT: Orginal (czerwony) i wynik filtracji (niebieski)'); pause
     n = 500:Nx-P;
     SNR_we_dB = 10*log10( sum( s(n).^2 ) / sum( (d(n)-s(n)).^2 ) ),
     SNR_wy_dB = 10*log10( sum( s(n).^2 ) / sum( (y(n)-s(n)).^2 ) ), pause
  end

end

if(ikrata==1)  % Czy chcemy filtrowac WIENER KRATOWO NIEadaptacyjnie: 0/1
    
% WIENER KRATOWO NIEADAPTACYNIE 
  [y,e] = filtrLWF(M,g,gamma,d,x);   % kratowo
  ty=dt*(0:length(y)-1); te=ty;

% Rysunki sygnalow wyjsciowych  
  figure
  subplot(211); plot(ty,y); grid; title('LATTICE: WY : sygnal y = dest');
  subplot(212); plot(te,e); grid; title('LATTICE: WY : sygnal e = err');
  xlabel('czas [sek]'); pause
  figure
  if (itest==1)
     subplot(111); plot(t,s,'r',te,e,'b'); grid; xlabel('czas [sek]');
     title('LATTICE: Orginal (czerwony) i wynik filtracji (niebieski)');
     n = 500:Nx;
     SNR_we_dB = 10*log10( sum( s(n).^2 ) / sum( (d(n)-s(n)).^2 ) ),
     SNR_wy_dB = 10*log10( sum( s(n).^2 ) / sum( (s(n)-e(n)).^2 ) ), pause
  end
  if (itest==2)
     n=1:Nx-P;
     subplot(111); plot(t(n),s(n),'r',ty(n),y(n),'b'); grid; xlabel('czas [sek]');
     title('LATTICE: Orginal (czerwony) i wynik filtracji (niebieski)');
     n = 500:Nx-P;
     SNR_we_dB = 10*log10( sum( s(n).^2 ) / sum( (d(n)-s(n)).^2 ) ),
     SNR_wy_dB = 10*log10( sum( s(n).^2 ) / sum( (y(n)-s(n)).^2 ) ), pause
  end   
   
end    
  
% ####################  
% WIENER ADAPTACZJNIE
% #################### 
  
% Startowe, przykladowe parametry filtracji
% M - dlugosc filtra
% L - liczba wektorow "przeszlych" w metodzie AP-LMS

mi=0; gamma=0; lambda=0; delta=0; alfa=0; L=0; leaky=0;

% !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
% WAZNE: USTAW PONIZEJ SAM WARTOSCI PARAMETROW ADAPTACJI
% !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

if( itest==1) % Adaptacyjne usuwanie interferencji 
    if(ialg==1)  mi=0.1; end                                           % LMS 0.1
    if(ialg==2)  mi=0.025; gamma=0.001; end                            % NLMS
    if(ialg==3)  L=4; mi=0.05; gamma=0.001; end                        % AP-LMS
    if(ialg==4)  mi=5; end                                             % zdekorelowany LMS
    if(ialg==5)  mi=1; gamma=0.001; end                                % zdekorelowany NLMS
    if(ialg==6)  mi=0.5; lambda=0.9999; alfa=1-lambda; delta=0.1; end  % LMS-Newton
    if(ialg==7)  lambda=0.9999; delta=0.5; end                         % WLS
    if(ialg==8) lambda=0.9999; delta=0.5; end                         % WRLS_standard
    if(ialg==9) lambda=0.9999; delta=0.5; end                         % WRLS_fast
    if(ialg==10) lambda=0.9999; delta=0.5; end                         % WRLS_Lattice
    if(ialg==11) lambda=0.9999; beta=1; end                            % GLWF
    if(ialg==12) M=64; mi=0.01; alfa=0.01; delta=0.25; gamma=0.00001;        end  % FDAF-overlap-add
    if(ialg==13) M=64; mi=0.05; alfa=0.01; delta=0.25; gamma=0.00001;        end  % FDAF-overlap-save
    if(ialg==14) Nh=64; M=16; mi=0.05; alfa=0.01; delta=0.25; gamma=0.00001; end  % FDAF-overlap-save part
end

if( itest==2) % Adaptacyjna poprawa linii telefonicznej
    if(ialg==1)  mi=0.01; end                                          % LMS
    if(ialg==2)  mi=0.1; gamma=0.001; end                              % NLMS
    if(ialg==3)  L=4; mi=0.05; gamma=0.001; end                        % AP-LMS
    if(ialg==4)  mi=0.05; end                                          % zdekorelowany LMS
    if(ialg==5)  mi=0.1; gamma=0.001; end                              % zdekorelowany NLMS
    if(ialg==6)  mi=0.1; lambda=0.9999; alfa=1-lambda; delta=0.1; end  % LMS-Newton
    if(ialg==7)  lambda=0.9999; delta=0.005; end                       % WLS
    if(ialg==8) lambda=0.9999; delta=0.005; end                       % WRLS_standard
    if(ialg==9) lambda=0.9999; delta=0.005; end                       % WRLS_fast
    if(ialg==10) lambda=0.99999; delta=0.0005; end                     % WRLS_lattice
    if(ialg==11) lambda=0.99; alfa=0.001; beta=alfa/(1-lambda); end    % GLWF
    if(ialg==12) M=64; mi=0.001; alfa=0.01; delta=0.25; gamma=0.00001;  end          % FDAF-overlap-add
    if(ialg==13) M=64; mi=0.001; alfa=0.01; delta=0.25; gamma=0.00001;  end          % FDAF-overlap-save
    if(ialg==14) Nh=256; M=64; mi=0.0001; alfa=0.0001; delta=0.0001; gamma=0.00001; end  % FDAF-overlap-save part
end

if( itest==3)  % Adaptacyjne identyfikacja obiektu 
    if(ialg==1)  mi=0.05; end                                          % LMS
    if(ialg==2)  mi=0.25; gamma=0.001; end                             % NLMS
    if(ialg==3)  L=4; mi=0.05; gamma=0.001; end                        % AP-LMS
    if(ialg==4)  mi=0.1; end                                           % zdekorelowany LMS
    if(ialg==5)  mi=1; gamma=0.001; end                                % zdekorelowany NLMS
    if(ialg==6)  mi=0.25; lambda=0.9999; alfa=1-lambda; delta=0.5; end % LMS-Newton
    if(ialg==7)  lambda=0.9; delta=0.1; end                            % WLS
    if(ialg==8) lambda=0.9; delta=0.1; end                            % WRLS_standard
    if(ialg==9) lambda=0.9; delta=0.1; end                            % WRLS_fast
    if(ialg==10) lambda=0.9; delta=0.1; end                            % WRLS_Lattice
    if(ialg==11) lambda=0.9; beta=1; end                               % GLWF
    if(ialg==12) M=32; mi=0.0001; alfa=0.01; delta=0.001; gamma=0.001;  end            % FDAF-overlap-add
    if(ialg==13) M=128; mi=0.001; alfa=0.001; delta=0.00001; gamma=0.00001;  end       % FDAF-overlap-save
    if(ialg==14) Nh=512; M=64; mi=0.001; alfa=0.01; delta=0.0001; gamma=0.00001; end  % FDAF-overlap-save part
end

if( itest==4)  % Adaptacyjne tlumienie echa 
    if(ialg==1)  M=20; mi=0.01; end                                    % LMS
    if(ialg==2)  M=20; mi=0.1; gamma=0.00001; end                      % NLMS
    if(ialg==3)  M=20; L=4; mi=0.001; gamma=0.00001; end               % AP-LMS
    if(ialg==4)  mi=0.1; end                                           % zdekorelowany LMS
    if(ialg==5)  mi=1; gamma=0.001; end                                % zdekorelowany NLMS
    if(ialg==6)  mi=0.25; lambda=0.9999; alfa=1-lambda; delta=0.5; end % LMS-Newton
    if(ialg==7)  M=20; lambda=0.9; delta=0.1; end                      % WLS
    if(ialg==8) M=10; lambda=0.9999; delta=0.001; end                 % WRLS_standard
    if(ialg==9) M=5; lambda=0.9999; delta=0.001; end                  % WRLS_fast
    if(ialg==10) M=50; lambda=0.999999; delta=0.0005; end              % WRLS_Lattice
    if(ialg==11) M=50; lambda=0.999999; beta=0.99; end                  % GLWF
    if(ialg==12) M=32; mi=0.0001; alfa=0.01; delta=0.001; gamma=0.001;  end            % FDAF-overlap-add
    if(ialg==13) M=128; mi=0.001; alfa=0.001; delta=0.00001; gamma=0.00001;  end       % FDAF-overlap-save
    if(ialg==14) Nh=4096; M=64; mi=0.001; alfa=0.01; delta=0.0001; gamma=0.00001; end  % FDAF-overlap-save part
end

% #####################
% Filtracja adaptacyjna
% #####################
  if(ialg<=6)  [y, e, h, hest ] = filtrNLMS(M,L,d,x,nr,mi,gamma,alfa,delta,leaky,ialg,ifig); end
  if(ialg==7)  [y, e, h, hest ] = filtrWLS(M,d,x,nr,lambda,ifig); end
  if(ialg==8)  [y, e, h, hest ] = filtrWRLS(M,d,x,nr,lambda,delta,1,ifig); end % przedostatni parametr: 0/1, 1=start macierz
  if(ialg==9)  [y, e, h, hest ] = filtrWRLS_faest(M,d,x,nr,lambda,delta); end
  if(ialg==10) [y, e, h, hest ] = filtrWRLS_lattice(M,d,x,nr,lambda); end
  if(ialg==11) [y, e, h, hest ] = filtrGLWF_ref(M,d,x,nr,lambda,beta); end
  if(ialg==12) [y, e, h, hest ] = filtrFDAF_overlap_add(M,d,x,nr,mi,alfa,delta,gamma); end
  if(ialg==13) [y, e, h, hest ] = filtrFDAF_overlap_save(M,d,x,nr,mi,alfa,delta,gamma); end
  if(ialg==14) [y, e, h, hest ] = filtrFDAF_overlap_save_part(Nh,M,d,x,nr,mi,alfa,delta,gamma); end  % mi=0.01, gamma=0.00001
 
  ty=dt*(0:length(y)-1);
  te=dt*(0:length(e)-1);
  [K,L] = size(h); th=dt*(0:K-1);

% ############################
% Rysunki sygnalow wyjsciowych
% ############################
  figure
  subplot(211); plot(ty,y); grid; title('ADAPT - WY : sygnal y = dest');
  subplot(212); plot(te,e); grid; title('ADAPT - WY : sygnal e = err');
  xlabel('czas [sek]'); pause
  figure
  subplot(211); plot(th,real(h(:,1))); grid; title('ADAPT - WY : h1[n]');
  subplot(212); plot(th,real(h(:,2))); grid; title('ADAPT - WY : h2[n]');
  xlabel('czas [sek]'); pause
  
  figure
  if (itest==1)
     subplot(111); plot(t,s,'r',te,e,'b'); grid; xlabel('czas [sek]');
     title('ADAPT: Orginal (czerwony) i wynik filtracji (niebieski)'); pause
     n = 500:Nx;
     SNR_we_dB = 10*log10( sum( s(n).^2 ) / sum( (d(n)-s(n)).^2 ) ),
     SNR_wy_dB = 10*log10( sum( s(n).^2 ) / sum( (s(n)-e(n)).^2 ) ), pause
  end
  if (itest==2)
     n=1:Nx-P;
     subplot(111); plot(t(n),s(n),'r',ty(n),y(n),'b'); grid; xlabel('czas [sek]');
     title('ADAPT: Orginal (czerwony) i wynik filtracji (niebieski)'); pause
     n = 500:Nx-P;
     SNR_we_dB = 10*log10( sum( s(n).^2 ) / sum( (d(n)-s(n)).^2 ) ),
     SNR_wy_dB = 10*log10( sum( s(n).^2 ) / sum( (y(n)-s(n)).^2 ) ), pause
  end
  if (itest==3)
     n=1:Nx-P;
     subplot(111); plot(t(n)',real(h(n,1)),'r',t(n)',real(h(n,2)),'b'); grid; xlabel('czas [sek]');
     title('ADAPT: h[1] (czerwony), h[2] (niebieski)'); pause
  end
 