% Przyklad 9.3:
% m-plik: przyklad_9_3.m
%
% Cyfrowe przetwarzanie sygnalow. Podstawy, multimedia, transmisja. PWN, Warszawa, 2014.
% Rozdzial 9: Przetworniki analogowo-cyfrowe i cyfrowo-analogowe.
% Autorzy rozdzialu: Wojciech Polchlopek, Roman Rumian.
%
% Wiecej na stronie internetowej: http://teledsp.kt.agh.edu.pl
%
% Opracowanie przykladu: marzec 2014 r./WP/RR.

clc; clear; close all;

w=16;                       % Dlugosc slowa w bitach (word length);
q=2^(1-w);                  % Krok kwantyzacji
N=1024;                     % Liczba probek;
n=0:N-1;                    % numery probek;     
k=2^6;                      % numer elementu DFT (bez przecieku);
Fo=k/N;                     % Unormowana czestotliwosc;

A=2^(-18);                  % Amplituda sygnalu (wersja 1);
%A=Q;                       % Amplituda sygnalu (wersja 2) - nalezy przeniesc znak komentarza powyzej;
x=A*sin(2*pi*n*Fo);         % Sygnal dyskretny;

% Rectangular dither - "dither" prostokatny:
r=rand(1,N);
r=r-mean(r);
r=r./max(abs(r));           % Normalizacja;
d_rect=r*q/2;
%Triangular Dither  - "dither" trojkatny:
r1=rand(1,N);
r1=r1-mean(r1);
r1=r1./max(abs(r1));
d_tri=(r+r1)*q/4;           % amplituda szumu dither;

xqt=floor(x/q)*q;           % Kwantyzacja z obcieciem 
et=x-xqt;

xqdt=floor((x+d_tri)/Q+0.5)*Q;       % Dither trojkatny
win=blackman(N)';           % Okno (wersja 1);
% win=hann(N)';             % Okno (wersja 2) - nalezy przeniesc znak komentarza powyzej;
win=N*win/sum(win);         % Skalowanie okna;
Xt=abs(fft(xqt.*win))/(N/2);

X2=abs(fft(xqdt.*win))/(N/2);
thres_dB=-140;
thres=10^(thres_dB/20);
F=(0:(N-1))/(N);

X2_dB=-140*ones(size(X2));
idx2=find(X2>thres);
X2_dB(idx2)=20*log10(X2(idx2)+eps);
X2_dB=20*log10(X2./0.5+eps);

Xt_dB=-140*ones(size(Xt));
idx4=find(Xt>thres);
Xt_dB(idx4)=20*log10(Xt(idx4)+eps);
Xt_dB=20*log10(Xt./0.5+eps);

figure(1)
subplot(211)
plot(xqt(1:100),'r');title('a)Kwantyzacja z obcieciem');hold on;
stem(x(1:100));hold off
xlabel('n \rightarrow'); 
ylabel(' x_q_t \rightarrow')

subplot(212)

plot(F,Xt_dB); title('b)Spektrogram dla obcinania');
axis([0 0.5 -140 -80]); 
grid on, xlabel('f/fp \rightarrow'); 
ylabel(' PDF{Q}(e^{j\Omega}) [dB] \rightarrow')

figure(2)

subplot(211)
plot(xqdt(1:100)); title('a)Dither trojkatny')
xlabel('n \rightarrow'); 
ylabel(' x_q_d_t \rightarrow')

subplot(212)
plot(F,X2_dB); title('b)Spektrogam dla trojkatnego dither`ingu');
axis([0 0.5 -140 -80]); 
grid on, xlabel('f/fp \rightarrow'); 
ylabel(' PDF{Q}(e^{j\Omega}) [dB] \rightarrow')
% 

SNR_t=10*log10(sum(x.^2)/sum(et.^2));

% Spektra z ditheringiem
figure(3)
Xdt=abs(fft(d_tri.*win))/(N/2);


Xdt_dB=-140*ones(size(Xdt));
idx7=find(Xdt>thres);
Xdt_dB(idx7)=20*log10(Xdt(idx7));
Xdt_dB=20*log10(Xdt./0.5);


subplot(211)

[n_tri,x_tri]=hist(d_tri,20);
bar(x_tri/Q,n_tri/sum(n_tri))

title('Histogram trojkatnego szumu dither');

subplot(212)

plot(F,Xdt_dB); title('Spektrum trojkatnego szumu dither');
axis([0 0.5 -140 -80]); 
grid on, xlabel('f/fp \rightarrow'); 
ylabel(' PDF{Q}(e^{j\Omega}) [dB] \rightarrow')

% KONIEC PRZYKLADU 9.3.