function [B] = compute_pdfs(o, hmm)
    % Argumenty wejsciowe
    % o - macierz obserwacji [T x D]
    % hmm - ukryty model Markowa 
    
    % Wartosci zwracane:
    % B - macierz [FxT] zawierajA...ca prawdopodobienstwa akustyczne p(o(t) | q(t) = f; hmm)
  
    [T, D] = size(o);
    [F, M, D2, D2_] = size(hmm.sigma);
    B = zeros(F, T);
  
    if (D~=D2)
       error('Dimensions mismatch!.')
    end
  
    % oblicz niezalezne od danych wartosci komponentow
    % aby nie robic tego samego dla kazdej z T obserwacji osobno
    g_const = zeros(F,M); inv_sigmas=zeros(F,M,D,D);
    for f=1:F
       for m=1:M
          sigma = reshape(hmm.sigma(f,m,:,:),D,D);
          g_const(f,m) = 1./sqrt((2*pi).^D*det(sigma));
          inv_sigmas(f,m,:,:) = inv(sigma);
       end
    end

    for t=1:T         %dla kazdej ramki
        for f=1:F     %dla kazdego ze stanow HMM
            for m=1:M %dla kazdego komponentu Gaussa w mieszaninie
                mu = reshape(hmm.mu(f,m,:),1,D);
                %sigma = reshape(hmm.sigma(f,m,:,:),D,D);   
                inv_sigma = reshape(inv_sigmas(f,m,:,:),D,D);
                exponent = (o(t,:)-mu)*inv_sigma*(o(t,:)-mu)';
                B(f,t) = B(f,t) + hmm.c(m).*g_const(f,m).*exp(-0.5*exponent);
                %B(f,t) = B(f,t) + hmm.c(m).*mvnpdf(o(t,:),mu,sigma);
            end
        end
    end   
end
