function [loglike, alphas, betas, gammas] = hmm_fwdback(priors, B, A, back_pass)
    % Argumenty wejsciowe:
    %  priors - wektor prawd. poczatkowych P(q=j|t=1)
    %  B = {b_j(o(t))}: p(o(t)|q=s_j;M), [FxT] macierz prawdopodobienstw z 
    % 			jakim dany stan q generuje obserwacje o(t)
    %  A = {a_ij}: P(q+1|q) macierz przejsc [FxF] modelu HMM
    %  back_pass - obliczac prawd. wstecz? (pomijane na etapie rozpoznawania)

    % Wartosci zwracane
    %  loglike - log p(O;M) - log prawdopodobienstwo calkowie z 
    % 		jakim dany model M generuje obserwacje O_1:T
    %  alphas - macierz [FxT] prawd. w przod
    %  betas - macierz [FxT] prawd. w tyl lub [] jezeli back_pass=false
    %  gammas = macierz [FxT] prawd. przynaleznosci stanu f do obserwacji o_t
    
    if (nargin==3) back_pass = false; end
    
    [F, T] = size(B);
    loglike = -Inf;
    alphas = zeros(F, T); ascales = zeros(1,T);  
    betas = []; bscales = [];
    gammas = [];
     
    % oblicz p(o(1),o(2),...,o(t),q;M)
    alphas(:,1) = priors.*B(:,1);
    [alphas(:,1) ascales(1)] = normalise(alphas(:,1));
    for t=2:T
        for f=1:F
            alphas(f,t) = alphas(:,t-1)'*A(:,f)*B(f,t);
        end
        [alphas(:,t), ascales(t)] = normalise(alphas(:,t));
    end

    loglike = sum(log(ascales));
        
    % obliczenie prawodpodobienstw backward jest opcjonalne
    if (back_pass == false) return; end

    betas = zeros(F, T); bscales = zeros(1, T);
    % oblicz p(o(t+1),o(t+2),...,o(t),q;M)
    betas(:,T) = ones(F, 1);
    bscales(T) = 1.0;
    for t=T-1:-1:1
        for f=1:F
            betas(f,t) = (betas(:,t+1).*B(:,t+1))'*A(:,f);
        end
        [betas(:,t), bscales(t)] = normalise(betas(:,t));
    end
    bscales(1) = sum((betas(:,1).*B(:,1))'*priors);
    beta_1=sum(log(bscales));
     
    %oblicz statystyki zajetosci stanow (rown. 14.51)
    gammas = zeros(F, T);
    for t=1:T
        [gammas(:,t), xxx] = normalise(alphas(:,t).*betas(:,t));
    end
end
