function [llike, best_path] = hmm_viterbi(priors, B, A)

    % Argumenty wejsciowe:
    %  priors - wektor prawd. poczatkowych P(q=j|t=1)
    %  B = {b_j(o_t)}: p(o_t|q_t=j;M), [QxT] macierz prawdopodobienstw z 
    %  jakim dany stan j modelu M generuje obserwacje o_t
    %  A = {a_ij}: P(q+1|q) macierz przejsc [QxQ] modelu HMM

    % Wartosci zwracane
    % llike = log max_Q p(O,Q;M) - log prawdopodobienstwo Viterbiego 
    %        najbardziej prawdopodobnej sekwencji stanow Q*
    % best_path - argmax_Q p(O,Q;M) - wektor o rozmiarze T z indeksami stanow

    [F, T]=size(B);
    llike = -Inf;
    deltas = zeros(F, T);
    psi = zeros(F,T);
    best_path = zeros(1, T);
    scales = ones(1,T);
 
    t=1;
    deltas(:,t) = priors.*B(:,t);
    [deltas(:,t), s] = normalise(deltas(:,t)); 
    scales(t) = 1/s;
    psi(:,t) = 0;
    for t=2:T
        for f=1:F
            [deltas(f,t), psi(f,t)] = max(deltas(:,t-1).*A(:,f));
            deltas(f,t) = deltas(f,t).*B(f,t);
        end
        [deltas(:,t), s] = normalise(deltas(:,t));
        scales(t) = 1/s;
    end

    [like, best_path(T)] = max(deltas(:,T));
 
    for t=T-1:-1:1 %sledzenie sciezki wstecz
        best_path(t) = psi(best_path(t+1), t+1);
    end

    llike = -sum(log(scales));
    
end
