function [G,mn,mx]=melbankf(J,n,fs,fl,fh)
%MELBANKF oblicza wagi filtrow trojkatnych dla banku filtrow z melowa skala
%         czestotliwosci. Wagi sa zapisywane w macierzy rzadkiej G
% 
% Argumenty wejsciowe:
%       J   liczba filtrow (okien czestotliwosciwoych) w banku filtrow 
%           pokrywajacych sie na polowie szerokosci w skali melowej
%		n   dlugosc fft
%		fs  czestotliwosc probkowania w Hz
%		fl  Czestotliwosc odpowiadajaca poczatkowi pasma najnizszego 
%           filtru, wyrazona jako ulamek fs, najczesciej przyjmuje sie fl=0
%		fh  Czestotliwosc odpowiadajaca koncowi pasma najwyzszego 
%           filtru, wyrazona jako ulamek fs, z reguly przyjmuje sie fh=0.5
%
% Argumenty wyjsciowe:	
%       G   macierz rzadka zawierajaca wagi filtrow trojkatnych. Rozmiar 
%           macierzy G wynosi: size(G)=[J,mx-mn+1]
%		mn  indeks pierwszego niezerowego prazka FFT
%		mx  indeks ostatniego niezerowego prazka FFT
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%-------------------------------------------------------------------------
%                          OBLICZANIE WAG FILTROW
%-------------------------------------------------------------------------

mflh=[fl fh]*fs;       % czestotliwosci graniczne w Hz
mflh=frq2mel(mflh);    % konwersja czestotliwosci granicznych do skali 
                       % melowej. Kod funkcji frq2mel znajduje sie w pliku
                       % frq2mel.m znajdujacym sie na stronie internetowej:
                       % http://www.teledsp.kt.agh.edu.pl/rozdzial_14
melrng=mflh(2)-mflh(1);% zakres czestotliwosci w melach
fn2=floor(n/2);        % indeks prazka FFT o najwiekszej dodatniej 
                       % czestotliwosci - czestotliwosc Nyquista jezeli n 
                       % jest parzyste
melinc=melrng/(J+1);   % odstep miedzy srodkowymi czestotliwosciami filtrow 
                       % w melach

% Obliczanie 4 prazkow FFT odpowiadajacych nastepujacym czestotliwosciom  
% blim = [poczatek_1-go_fitru    srodek_1-go_fitru     srodek_p-tego_fitru
% koniec_p-tego_filtru]. Kod funkcji mel2freq znajduje sie w pliku
% mel2freq.m znajdujacym sie na stronie internetowej:
% http://www.teledsp.kt.agh.edu.pl/rozdzial_14
blim=mel2frq(mflh(1)+[0 1 J J+1]*melinc)*n/fs;

mc=mflh(1)+(1:J)*melinc;       % srodkowe czestotliwosci filtrow w melach
b1=floor(blim(1))+1;           % nr wymaganego do obliczen prazka FFT o 
                               % najmniejszym indeksie (moze byc ujemny)
b4=min(fn2,ceil(blim(4))-1);   % nr wymaganego do obliczen prazka FFT o  
                               % najwiekszym indeksie

% rozmieszczanie wszystkich uzytecznych prazkow FFT wzgledem srodkow
% filtrow
pf=(frq2mel((b1:b4)*fs/n)-mflh(1))/melinc;

%  usuwanie wszystkich nieprawidlowych wartosci z wektora pf, ktore moga
%  sie pojawic na skutek bledow zaokraglen
if pf(1)<0      pf(1)=[];   b1=b1+1;    end
if pf(end)>=J+1 pf(end)=[]; b4=b4-1;    end

fp=floor(pf);            % Wektor na podstawie ktorego mozna przypisac 
                         % prazki FFT do filtrow, prazek i nalezy do 
                         % filtrow o numerach fp(1+i-b1)+[0 1]
pm=pf-fp;                % mnozniki dla filtru o wyzszym numerze dla danego 
                         % prazka FFT
k2=find(fp>0,1);         % k2+b1 jest indeksem pierwszego prazka FFT, 
                         % nalezacego do dwoch filtrow 
k3=find(fp<J,1,'last');  % k2+b1 jest indeksem ostatniego prazka FFT, 
                         % nalezacego do dwoch filtrow
k4=numel(fp);            % k4+b1 jest ostatnim prazkiem, ktory nalezy do 
                         % jakiegokolwiek filtru
if isempty(k2)
    k2=k4+1;
end
if isempty(k3)
    k3=0;
end

r=[1+fp(1:k3) fp(k2:k4)];% indeksy rzedow w macierzy rzadkiej G. Indeks 
                         % rzedu odpowiada numerowi filtra trojkatnego
c=[1:k3 k2:k4];          % indeksy kolumn w macierzy rzadkiej G. 
                         % Indeks kolumny odpowiada numerowi prazka FFT
v=[pm(1:k3) 1-pm(k2:k4)];% Wartosci wag filtrow trojkatnych zapisywanych 
                         % w macierzy rzadkiej G. Do komorki macierzy G o
                         % indeksach [r(k),c(k)] jest zapisywana wartosc
                         % v(k) -> G(r(k),c(k))=v(k).
mn=b1+1; % obliczenie indeksu pierwszego niezerowego prazka FFT
mx=b4+1; % obliczenie indeksu ostatniego niezerowego prazka FFT
    
if b1<0
    c=abs(c+b1-1)-b1+1;   % konwersja ujemnych indeksow prazkow na dodatnie
end

G=sparse(r,c,v); % zapisywanie wag filtrow do macierzy rzadkiej

%-------------------------------------------------------------------------
%                        WIZUALIZACJA
%-------------------------------------------------------------------------
% rysowanie wykresow wag filtrow z banku filtrow w przypadku braku
% argumentow wyjsciowych
%
if ~nargout % rysowanie idealizowanych ch-k czestotliwosciowych filtrow
    ng=201;     % 201 punktow
    me=mflh(1)+(0:J+1)'*melinc;
 
   fe=mel2frq(me); % definiowanie czestotliwosci
   xg=mel2frq(repmat(linspace(0,1,ng),J,1).*repmat(me(3:end)-me(1:end-2),...
       1,ng)+repmat(me(1:end-2),1,ng));
     v=1-abs(linspace(-1,1,ng));

    v=repmat(v,J,1);
    plot(xg',v','b');
    set(gca,'xlim',[fe(1) fe(end)]);
    xlabel(['Czestotliwo?c [Hz]']);
    pause
end