function [c] = melcepstrum(s,fs,w,nc,p,n,inc,fl,fh)

%MELCEPSTRUM Oblicza melowe wspolczynniki cepstralne (MFCC) sygnalu,
%     okienkowanie sygnalu w dziedzinie czasu jest dokonywane za pomoca
%     okna Hamminga. Przyjety zostal trojkatny ksztalt filtrow z banku
%     filtrow w dziedzinie melowej.
%
% argumenty wejsciowe:
%     s	 sygnal mowy
%     fs  czestotliwosc probkowania w Hz
%     nc  liczba wspolczynnikow MFCC z wylaczeniem zerowego wspolczynnika
%     n   dlugosc ramki w probkach 
%     p   liczba filtrow w banku filtrow
%     inc przesuniecie miedzy poczatkami ramek w probkach
%     fl  dolny kraniec pierwszego filtra z banku filtrow wyrazony jako 
%         ulamek fs 
%     fh  gorny kraniec ostatniego filtra z banu filtrow wyrazony jako 
%         ulamek fs
%
%     w   jezeli jest ustawiona wartosc w='0', to zostaje uwzgledniony
%         zerowy wspolczynnik cepstralny
%
% argumenty wyjsciowe:	
%      c    macierz wspolczynnikow MFCC, indeks rzedu macierzy odpowiada
%           indeksowi ramki
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

z=ramkowanie(s,hamming(n),inc);   %podzial sygnalu na ramki z uzyciem okna 
                                % Hamminga
f=rfft(z.');                    % obliczanie fft we wszystkich ramkach 
                                % sygnalu. Kod funkcji rfft znajduje sie w 
                       % pliku rfft.m znajdujacym sie na stronie 
                       % internetowej:
                       % http://www.teledsp.kt.agh.edu.pl/rozdzial_14     
[m,a,b]=melbankf(p,n,fs,fl,fh);% wywolanie funkcji obliczajacej wagi 
                                % filtrow (trojkatnych)wagi sa zapisywane 
                                % w macierzy rzadkiej m
pw=f(a:b,:).*conj(f(a:b,:));    % obliczanie kwadratu widma amplitudowego 
                                % dla wszystkich ramek sygnalu
pth=max(pw(:))*1E-20;           % obliczanie minimalnej wartosci jaka moze 
                                % byc logarytmowana
y=log(max(m*pw,pth));           % obliczanie zlogarytmowanych 
                                % wspolczynnikow widmowych. 
c=rdct(y).'; % wykonanie dct na zlogarytmowanych wspolczynnikach widmowych
             % Kod funkcji rdct znajduje sie w pliku rdct.m znajdujacym sie
             % na stronie internetowej:
             % http://www.teledsp.kt.agh.edu.pl/rozdzial_14
nf=size(c,1);   % liczba ramek sygnalu
nc=nc+1;
c(:,nc+1:end)=[];

if ~any(w=='0') % w przypadku, gdy nie jest uwzgledniany zerowy 
                % melowy wspolczynnik cepstralny
   c(:,1)=[];
   nc=nc-1;
end

%-------------------------------------------------------------------------
%                        WIZUALIZACJA
%-------------------------------------------------------------------------
% Wizualizacja wspolczynnikow MFCC w przypadku braku argumentow wyj?ciowych


if nargout<1
   [nf,nc]=size(c);
   t=((0:nf-1)*inc+(n-1)/2)/fs;
   ci=(1:nc)-any(w=='0');
   imh = imagesc(t,ci,c.');
   axis('xy');
   xlabel('Czas (s)');
   ylabel('Numer wspolczynnika MFCC');
   map = (0:63)'/63;
   colormap([map map map]);
   colorbar;
end

