% Przyklad 14.05
% Trenowanie izolowanych modeli HMM dla cyfr od 0 do 9

function [] = trenuj_HMM_melcepstrum(experiment_name)
    
    % zakladamy, ze sygnal zostal zparametryzowany i zapisany w
    % katalogu eksperymentu
    load(strcat(experiment_name, '/uczacy_melcepstrum.mat'));
    uczacy_feats = melfeats;
    [num_classes, num_examples_train] = size(uczacy_feats);

    F=3;                        % liczba stanow modelu HMM
    D=size(uczacy_feats{1}, 2); % wymiarowosc cech
    priors = [1 zeros(1, F-1)]';% prawdopodobienstwa poczatkowe
    maxiter = 10;               % maksymalna liczba iteracji EM
    
    tic();
    
    % utworz zbior num_classes modeli HMM (kazdy opisuje pojedyncza cyfre)
    % majacych F stanow modelowanych przez pojedynczy komponent Gaussa o D wymiarach
    hmms = hmm_create(num_classes, F, 1, D);
    
    % zainicjalizuj poczatkowe wartosci rozkladow srednia i wariancja danej klasy
    for i=1:num_classes
        fprintf(1,'---- Initialising an HMM for the word %d \n',  i-1)
        [gmean, gstdev] = compute_cmvn_stats(melfeats(i,:));
        hmms(i) = hmm_init(gmean, gstdev, hmms(i));
    end

    % estymuj parametry kazdego z modeli HMM za pomoca algorytmu EM
    for i=1:num_classes
        fprintf(1,'---- Training the HMM for the word %d \n',  i-1)
        [hmms(i), llike] = hmm_em(uczacy_feats(i,:), priors, hmms(i), maxiter);
    end
    
    fprintf(1,'Training of HMMs took %f seconds.\n', toc());
    
    %zapisz modele na dysku
    save_filename = strcat(experiment_name, '/', 'hmm_models.mat' );
    if is_octave()
        save('-mat', save_filename, 'hmms')
    else
        save(save_filename, 'hmms', '-mat')
    end

end


