function dct_proc(img_in, proc),
% dct_proc(img_in, proc),
%
% Funkcja oblicza transformate DCT dla calego obrazu, a nastepnie odtwarza obraz z zadanej liczby wspolczynnikow transformaty.
% img_in - obraz wejsciowy (monochromatyczny);
% proc - procent wspolczynnikow transformaty (0-100%) uzytych do odtworzenia obrazu;
%
% Przyklad 18.2.
% m-plik (funkcyjny): dct_proc.m
%
% Cyfrowe przetwarzanie sygnalow. Podstawy, multimedia, transmisja. PWN, Warszawa, 2014.
% Rozdzial 18. Kompresja obrazow. 
% Autor rozdzialu: Marek Domanski.
%
% Wiecej na stronie internetowej: http://teledsp.kt.agh.edu.pl
%
% Opracowanie przykladu: kwiecien 2014 r./MD.

clc;

if (proc < 0 || proc > 100),
    fprintf('Wartosc spoza zakresu 0 - 100 %% \n');
end;

dim = size(img_in);
img_dct=dct2D(img_in);
out_dct=zeros(dim(1),dim(2));
x=ceil(sqrt(dim(1)*dim(2)*(proc/100)));
out_dct(1:x,1:x)=img_dct(1:x,1:x);
out_img=idct2D(out_dct);

% przedstawienie wynikow:
    figure(1);
        subplot(2,3,1); 
            imagesc(img_in, [0 255]);        colormap(gray);        axis image;
                title('Obraz oryginalny');
        subplot(2,3,2); 
            imagesc(abs(img_dct), [0 255]); axis image;
                title('Widmo obrazu oryginalnego');
        subplot(2,3,3); 
            imagesc(uint8(abs(double(img_in)-out_img)), [0 255]);   axis image;
                title('Roznica obrazow: oryginal-rekonstrukcja');
        subplot(2,3,4); 
            imagesc(out_img, [0 255]);                              axis image;
                title('Obraz zrekonstruowany');
        subplot(2,3,5); 
            imagesc(abs(out_dct), [0 255]);                         axis image;
                title('Widmo obrazu zrekonstruowanego');
        subplot(2,3,6); 
            imagesc(uint8(abs(double(img_in)-out_img)));            axis image;
                title('Przeskalowana roznica obrazow');

%--------------------------------------
% Funkcje lokalne
%--------------------------------------

function b=dct2D(arg1),
% b=dct2D(arg1),
%
% DCT2 2-D. Wyznacza dwuwymiarowa transformate kosinusowa macierzy A.
% B = DCT2D(A)

[m, n] = size(arg1);
if (m > 1) && (n > 1),
    b = dct(dct(arg1).').';
else
    error('Rozmiar macierzy co najmniej 2x2');
end;
%--------------------------------------

function a = idct2D(arg1),
% a = idct2D(arg1),
%
% IDCT2 2-D . Wyznacza odwrotna dwuwymiarowa transformate kosinusowa macierzy A.
% B = IDCT2D(A)

[m, n] = size(arg1);
if (m > 1) && (n > 1),
    a = idct(idct(arg1).').';
else
    error('Rozmiar macierzy co najmniej 2x2');
end;

% KONIEC PRZYKLADU 18.2.