
% Testowanie metod estymacji czestototliwosciowej, opisanych w rozdziale 20.
% Autor: Tomasz ZIELINSKI, tzielin@agh.edu.pl

close all, clear all,

%% Parametry sygnalow
N = 500;     % liczba probek sygnalu
K = 3;       % liczba skladowych rzeczywistych, od 1 do 4
SNR = 160;   % stosunek sygnal/szum [decybele]

Om = [12.345 77.89 111.22 145.67 ]*(2*pi)/N,  % pulsacje - numery prazkow DFT
d  = [0.001 0.01 0.10 0.20 ],                 % tlumienia % WAZNE: dla fPisarenko i fESPRIT jest wymagany brak tlumienia
% d = [ 0 0 0 0 ];                            % fPisarenko i fESPRIT; UWAGA: fKT niestabilne dla d = 0
A = [4.3  3.3  2.3  1.3  ];                   % amplitudy
p = [-pi/11 -pi/7 -pi/5 -pi/3];               % fazy


%% Generacja sygnalu testowego
n = 0:N-1;
x = zeros(1,N);
for k=1:K
    x = x + A(k)*cos(Om(k)*n+p(k)).*exp(-n*d(k));
end
y = awgn( x, SNR) ;
plot(y); title('Analizowany sygnal'); xlabel('n'); grid; pause

%% Dekompozycja EVD

L = 2*K+20;                    % wymiar macierzy autokorelacji 
ryy=xcorr(y,'unbiased');       % do wyboru: 'unbiased', 'biased', 'coeff', 'none'
ryy = ryy(N:end);              % z powodu symetrii, bierzemy tylko polowe
Ryy = toeplitz( ryy(1:L+1) );  % zbudowanie macierzy Ryy
[U, D ] = eig( Ryy );          % EVD( Ryy )
sigma = diag( D );             % wartosci wlasne
[sigma, indx] = sort( sigma, 'descend' );
stem( sigma ), title('Warosci wlasne'); grid, pause
subplot(611); plot(U(:,indx(1))), title('Wektor wlasny 1'); grid,
subplot(612); plot(U(:,indx(2))), title('Wektor wlasny 2'); grid,
subplot(613); plot(U(:,indx(3))), title('Wektor wlasny 3'); grid,
subplot(614); plot(U(:,indx(4))), title('Wektor wlasny 4'); grid,
subplot(615); plot(U(:,indx(5))), title('Wektor wlasny 5'); grid,
subplot(616); plot(U(:,indx(6))), title('Wektor wlasny 6'); grid,
pause

%% Wyniki analizy
if(K==1)
  disp(' 1. fHilbert #############'); [ Om, D ] = fHilbert(x,K),    % K=1 okno Hanninga, K=2 Blackmana
end
disp('  2. fProny ###############');  [ Om, D ] = fProny(x,K),      %
disp('  3. flpCOR 1 #############');  [ Om, D ] = flpCOR(x,K,1),    % metoda 1
disp('  4. flpCOR 2 #############');  [ Om, D ] = flpCOR(x,K,2),    % metoda 2
disp('  5. flpCOR 3 #############');  [ Om, D ] = flpCOR(x,K,3),    % metoda 3
disp('  6. fSTMB ################');  [ Om, D ] = fSTMB(x,K,10),    % 10 = liczba iteracji
disp('  7. fKT ##################');  [ Om, D ] = fKT(x,K),         %
disp('  8. fMatPen ##############');  [ Om, D ] = fMatPen(x,K),     %
disp('  9. fPisarenko ###########');  [ Om    ] = fPisarenko(x,K),  %
disp(' 10. fESPRIT ##############');  [ Om    ] = fESPRIT(x,K),     %
 