
% Demodulacja sygnalu za pomoca filtru Hilberta (przesuwnik fazowy o -90 stopni)
% Wersja 1
% Autor: Tomasz ZIELINSKI, tzielin@agh.edu.pl

clear all; close all;

fpr = 1000;

% Filtr Hilberta

% Odpowiedz impulsowa
M=200; N=2*M+1;
n = -M:M;
h = (1-cos(pi*n)) ./ (pi*n); h(M+1)=0;
stem(n, h); title('h[n]'); pause

% Ch-ka czestotliwosciowa
f = 0:0.5:fpr/2;
z = exp(j*2*pi*f/fpr);
H = polyval(h,z); 
plot(f,20*log10(abs(H))); xlabel('f [Hz]'); grid; title('H(f)');pause

% Okno
% w=boxcar(N)';
% w = kaiser(N,5)';
w = chebwin(N,100)';
stem(w); pause
f = 0:0.5:fpr/2;
z = exp(j*2*pi*f/fpr);
W = polyval(w,z);
plot(f,20*log10(abs(W)/abs(W(1)))); xlabel('f [Hz]'); grid; title('H(f)');pause

% Odpowiedz impulsowa z oknem
h = h .* w;
stem(h); pause
f = 0:0.5:fpr/2;
z = exp(j*2*pi*f/fpr);
H = polyval(h,z);
plot(f,20*log10(abs(H))); xlabel('f [Hz]'); grid; title('H(f)');pause

% #################
% Czy filtr dziala?
% #################

% Generacja sygnalu
Nx = 1000; nx=0:Nx-1;
dt=1/fpr; t=dt*(0:Nx-1);

% FM Modulacja czestotliwosci
f0=200; df = 150; fm1 = 2;
%f = f0 + df*t;
%fi = 2*pi*(f0*t+0.5*df*t.^2);
 f = f0 + df*sin(2*pi*fm1*t);
 fi = 2*pi*(f0*t-df/(2*pi*fm1)*cos(2*pi*fm1*t));

% AM modulacja amplitudy
A = 100; dA=0.5; fm2=2;
 obw = A * (1 + dA*sin(2*pi*fm2*t));
%obw = A * exp(-100*(t-0.5).^2);
%obw = A * ones(1,Nx);

x = obw .* cos(fi);

% Filtracja
bx=zeros(1,N);
for n=1:Nx
    bx = [ x(n) bx(1:N-1) ];
    y(n) = sum( bx.*h );
end    

nh = M+1:Nx-M;
y = y(N:end);
x = x(nh);

z = x + j*y;

% Odtworzenie amlitudy
obw_e = abs(z);
plot(t(nh),x,'r',t(nh),y,'b',t(nh),obw_e,'g'); pause
plot(x,y); pause
plot(t(nh),obw(nh),'r',t(nh),obw_e,'b'); title('AM'); pause

% Odtworzenie czestotliwosci
fi_e = unwrap(angle(z));
w_e = diff(fi_e)./ diff(t(nh));
f_e = w_e/(2*pi); f_e = [ 0 f_e];
plot(t(nh),f(nh),'g',t(nh),f_e,'b'); title('FM'); pause

% Widmo sygnalu analitycznego
Ny = length(y);
w = chebwin(Ny,140)';
k = 0:Ny-1;
df=fpr/Ny; f=df*k;
Z = 20*log10(abs(fft(z.*w)));
X = 20*log10(abs(fft(x.*w)));
subplot(211); plot(f,Z(k+1)); xlabel('f[Hz]'); title('Z(f)'); grid;
subplot(212); plot(f,X(k+1)); xlabel('f[Hz]'); title('X(f)'); grid; pause




