
% ############################################
% RADIO FM: DEKODER - przyklad
% Dekoder RDS wedlug standardu IEC 62106
% Program fm_dekoder.m
% Autor: Tomasz ZIELINSKI, tzielin@agh.edu.pl

  fm_param   % wczytaj parametry

% Okreslenie danych wejsciowych 
% y = x; clear x;
  [y,fs,Nbits] = wavread('FM_Radio_IQ_250KS.wav');         % sygnal zsyntezowany
% [y,fs,Nbits] = wavread('FM_Radio_Krakow_101600kHz_IQ_250KS.wav',25*fs); % SDR#
  y = y(:,1) + sqrt(-1)*y(:,2);                            % IQ --> complex

% ODTWORZENIE SYGNALU MONO
% Obliczenie czestotliwosci chwilowej - demodulacja czestotliwosciowa
  dy = (y(2:end).*conj(y(1:end-1)));
  y = atan2( imag(dy), real(dy) ); clear dy;
% Filtracja dolnoprzepustowa sygnalu L+R (mono)
  ym = filter( hLPaudio, 1, y );
% Pozostawienie co fs/Abw-tej probki
  ym = ym(1:fs/Abw:end);
% Odsluchanie sygnalu L+R (mono) i zapisanie go na dysk
  soundsc(ym,Abw);
  w = ym-mean(ym); w=w/max(abs(w)); wavwrite(w,Abw,'FM_Radio_mono.wav' ); clear w;

% ODTWORZENIE SYGNALU STEREO
% Odseparowanie sygnalu pilota 19 kHz (filtracja pasmowoprzepustowa)
  p = filter(hBP19,1,y);
% Petla PLL z filtrem typu IIR do odtworzenia czestotliwosci i fazy pilota [7]
% i na tej podstawie sygnalow nosnych: symboli c1, stereo c38 i danych RDS c57
  freq = 2*pi*fpilot/fs; theta = zeros(1,length(p)+1);
  alpha = 1e-2; beta = alpha^2/4;
  for n = 1 : length(p)
      perr = -p(n)*sin(theta(n));
      theta(n+1) = theta(n) + freq + alpha*perr;
      freq = freq + beta*perr;
  end
  c1(:,1) = cos(theta(1:end-1)/16+phasePSF);               % nosna 1,1875 kHz
  c38(:,1) = cos(2*theta(1:end-1));                        % nosna 38 kHz
  c57(:,1) = cos(3*theta(1:end-1)); clear p; clear theta;  % nosna 57 kHz
% Filtracja pasmowo-przepustowa sygnalu L-R wokol 38 kHZ
  ys = filter(hBP38,1,y);
% Przesuniecie sygnalu L-R w czestotliwosci do DC (38 kHz --> 0 kHz)
  ys = real(ys .* c38); clear c38;
% Filtracja dolno-przepustowa
  ys = filter( hLPaudio, 1, ys );
% Pozostawienie co fs/Abw-tej probki
  ys = ys(1:fs/Abw:end);
% Synchronizacja czasowa sygnalow L+R i L-R (uwzglednienie opoznienia L-R)
  delay = (L/2)/(fs/Abw); ym = ym(1:end-delay); ys=2*ys(1+delay:end); % 2 od modul
  clear ymm yss;
% Odtworzenie kanalow L i R
  y1 = 0.5*( ym + ys ); y2 = 0.5*( ym - ys ); clear ym ys;
% De-emfaza, ch-ka plaska do 2.1 kHz, potem opadanie 20 dB na dekade
  y1 = filter(b_de,a_de,y1); y2 = filter(b_de,a_de,y2);
% Odsluchanie sygnalu stereo po de-emfazie
  soundsc([y1' y2'],Abw); pause
% Zapisanie sygnalu stereo radia FM na dysk
  maxi = max( max(abs(y1)),max(abs(y2)) );
  wavwrite( [ y1/maxi y2/maxi ], Abw, 'FM_Radio_stereo.wav' ); clear y1 y2;

% ZDEKODOWANIE BITOW SYGNALU RDS
% Filtracja pasmowo-przepustowa wokol czestotliwosci 57 kHz
  y = filter(hBP57,1,y);
% Przesuniecie sygnalu RDS w czestotliwosci do DC (57 kHz --> 0 kHz)
  y = y .* c57; clear c57;
% Filtracja LowPass dopasowana - filtr ksztaltujacy impulsy RDS
  y = filter( hpsf, 1, y );
% Synchronizacja symbolowa (timing recovery)
  n1 = 5000; Nmany = 50*210;
  for n = 1:2*210-1
      synchro(n) = sum( ( y(n1+n-1:n1+n-1+Nmany-1).*c1(n1 : n1+Nmany-1) ) );
  end
  [v, indx] = max(synchro);
  y = y(indx:end); c1=c1(1:end-indx+1); % zsynchronizowanie sygnalu z "zegarem"
% Przesuniecie sygnalu w czestotliwosci do DC (1.1875 kHz --> 0 kHz)
  y = y .* c1; y = y/max(y);
% Proste calkowanie - suma biezaca ostatnich fs/Tsymb probek
  h = ones(1,round(fs/fsymb)); y = filter(h,1,y); y = y/max(y);
% Znalezienie ujemnego i dodatniego zbocza "zegara"
  N = length(c1); M=5; M2=(M-1)/2; maxi1 = []; maxi2 = [];
  for n = M2+1:N-M2
     if( (c1(n-2)>0) & (c1(n-1)>0) & (c1(n+1)<0) & (c1(n+2)<0) ) % ujemne zbocze
         maxi1 = [ maxi1 n ];
     end
     if( (c1(n-2)<0) & (c1(n-1)<0) & (c1(n+1)>0) & (c1(n+2)>0) ) % dodatnie zbocze
         maxi2 = [ maxi2 n ];
     end
  end
  if( std(y(maxi1)) > std(y(maxi2)) ) maxi=maxi1; else maxi=maxi2; end % DD
% Zmiana poziomow: {-1, +1} --> {0,1}
  rdsbits = (-sign( y( maxi ) )+1)/2; clear c1 maxi maxi1 maxi2;
% Dekodowanie roznicowe z uwzglednieniem bi-fazowosci sygnalu
  rdsbits = abs(rdsbits(2:end) - rdsbits(1:end-1)); rdsbits = rdsbits(2:2:end)
% Zapisanie odczytanych bitow na dysk - dla programu rds.m L. Hollevoeta
  save FM_Radio_RDS.txt rdsbits /ascii
  