
% Przyklad 22.6
% Autorzy: Sebastian LIS, Jacek MAJERCZYKI (wyrazili zgode na publikacje),
%          Tomasz ZIELINSKI (tzielin@agh.edu.pl)
  clear all; close all;

% Parametry
  Nsym = 100;       % liczba probek na symbol
  Nmod = 10;        % liczna probek na nosna
  fsym = 1;         % liczba symboli na sekunde
  Kramek = 3;       % liczba wyslanych ramek/blokow symboli
  Npsf = 1001;      % liczba wspolczynnikow filtra ksztaltujacego impuls; maks 1601
  snr = 160;        % stosunek sygnalu do szumu
  fpr = Nsym*fsym;  % czestotliwosc probkowania
  wzm = 0.1;        % wzmocnienie kanalu
  faza = pi/3;      % przesuniecie fazowe kanalu
  kanal = 1;        % znieksztalcenie wprowadzane przez kanal: 0/1 NIE/TAK
  korektor = 1;     % korektor kanalu: 0/1 NIE/TAK
  dane = [ 0 1 0 2 3 0 3 1 1 2 1 0 2 2 0 1 3 3 0 2 1 1 0 3 0 2 1 1 3 1 0 2 ];
  synch = [ 3 0 0 1 2 1 3 0 3 2 2 1 3 0 0 1 2 1 3] ; % wzorzec synchro typu STS

% KODER: generowanie zespolnonych symboli QPSK
% nr symbolu = bity: 0=11 1=01 2=00 3=10
% odpowadajace fazie: pi/4 3pi/4 7pi/4 5pi/4
% odpowiadajace complex: 1+j -1+j -1-j 1-j
% koder = (1/sqrt(2))* [ 1+j, -1+j, -1-j, 1-j ];
  koder = [ exp( j*pi/4 ) exp( j*3*pi/4 ) exp( j*7*pi/4 ) exp( j*5*pi/4 ) ];
  psf = firrcos( Npsf-1, fsym/2, 0.35, fpr,'rollof', 'sqrt');  % filtr ksztaltujacy
  psf1 = sqrt(2)*(fpr/fsym)*psf; psf2 = 2*psf;                 % pulse shaping filter
  plot(psf1'); xlabel('n'); title('psf[n]'); grid; pause

% NADAJNIK #########################################################################
  
  ramka = [ synch dane ];                  % pojedynczna wyslana ramka symboli
  wyslane = [ 0 ramka ramka ramka ];       % wyslana sekwencja numerow stanow
  Ksym = length(ramka);                    % liczba symboli w jednej ramce
  Nramka = Ksym*Nsym;                      % liczba probek sygnalu na jedna ramke symboli
  IQwyslane = koder( wyslane+1 );          % numery symboli --> stany konstelacji
  sygnal = upfirdn(IQwyslane,psf1,Nsym,1); % nadprobkowanie Nsym razy z uzyciem psf
  Nsygnal = length(sygnal);
% Rysunki
  L = 10000;                               % ile probek sygnalu pokazac
  figure                                   % wykres fazorowy: imag=funkcja(real)
  plot(real(sygnal(1:L)),imag(sygnal(1:L)),'b-');
  title('Q=f(I)'); xlabel('I[n]'); ylabel('Q[n]'); pause
  figure                                   % real, imag
  subplot(311); plot(real(sygnal(1:L))); title('I[n]');
  subplot(312); plot(imag(sygnal(1:L))); title('Q[n]');
  subplot(313); plot( real(sygnal(1:L)).^2 + imag(sygnal(1:L)).^2 ); title('abs(IQ)');
  pause
  
% Modulacja - przejscie na wyzsza czestotliwosc
  n = 0:Nsygnal-1;
  sygnal = real(sygnal).*cos(2*pi/Nmod*n) - imag(sygnal).*sin(2*pi/Nmod*n);
    
% KANAL ############################################################################
  if(kanal==1) sygnal = sygnal*wzm*exp(-j*faza); end % tlumienie i opoznienie
  sygnal = awgn( sygnal, snr ); % dodanie szumu
  
% ODBIORNIK ########################################################################

% Demodulacja - powrot na niska czestotliwosc, eliminacja wplywu kanalu
  sreal = cos(2*pi/Nmod*n) .* sygnal;
  simag = -sin(2*pi/Nmod*n) .* sygnal;
  sygnal = sreal + j*simag;
  
% Korektor FEQ  
  if(kanal==1 & korektor==1) sygnal = sygnal*(1/wzm)*exp(j*faza); end
  
% Generacja sygnalu do synchronizacji
  IQsynch = koder( synch+1 );                % numery symboli --> stany konstelacji
  sync = upfirdn( IQsynch, psf1, Nsym, 1 );  % nadprobkowanie Nsym razy z uzyciem psf
  Nsync = length(sync);

% Splot z filtrem ksztaltujacym
  Nstart = (Npsf+1)/2;
  sygnal= conv(sygnal,psf2); Nsygnal = length(sygnal);
  sync = conv(sync,psf2); Nsync = length(sync);
  Nstart = Npsf;                             % aktualizacja przesuniecia analizy
  
% Rysunki
  figure                                     % real, imag
  subplot(211); plot(real(sygnal(1:L))); title('I[n]');
  subplot(212); plot(imag(sygnal(1:L))); title('Q[n]'); pause
  figure                                     % wyres fazorowy: imag=f(real)
  plot(sygnal(1:L),'b-'); title('Q=f(I) demod'); xlabel('I[n]'); ylabel('Q[n]'); pause

% Synchronizacja (przyklad)
  korel = abs( conv( fliplr(conj(sync)),sygnal) );  % korelacja sygnalu i synchro.
  [xmax, imax] = max( korel( 1:round(Nramka) ) );   % znalezienie pierwszego maksimum
  korel = korel / max( korel );                     % normalizacja
  iwork = Nstart+round(Nsym);
  iback = (Nsync+round(Nsym))-iwork;
  nfirst = imax-iback-1;                            % probka srodka pierwszego symbolu
  figure
  n = nfirst + (0 : Ksym*Kramek-1)*Nsym;
  plot( real(sygnal), imag(sygnal),'b-', real(sygnal(n)), imag(sygnal(n)),'ro');
  title('Q=f(I)'); xlabel('I[n]'); ylabel('Q[n]'); pause

% Detekcja
  for k = 0 : Kramek-1
     k
     detekt = [];
     num = nfirst + k*Nramka + round((0:Ksym-1)*Nsym); % polozenia symboli
     I = real(sygnal(num));
     Q = imag(sygnal(num));
     for m=1:length(num)
         if( I(m)>0 & Q(m)>0 ) detekt = [detekt 0]; end
         if( I(m)<0 & Q(m)>0 ) detekt = [detekt 1]; end
         if( I(m)>0 & Q(m)<0 ) detekt = [detekt 2]; end
         if( I(m)<0 & Q(m)<0 ) detekt = [detekt 3]; end
         if( (I(m)==0) | (Q(m)==0) ) disp('ZERO'); pause; end
     end
     ramka, detekt,
     if( sum( abs( ramka - detekt ) ) == 0 ) disp('TO SAMO !!!'); pause;
     else disp('BLAD !!!'); pause;
     end
  end
  