
% Przyklad 22.6b
% TESTOWANIE MODULACJI pi/4-DQPSK (TETRA)
% Autorzy: Sebastian LIS, Jacek MAJERCZYKI (wyrazili zgode na publikacje),
%          Tomasz ZIELINSKI (tzielin@agh.edu.pl)

clear all; close all;

% Parametry
  Nsym = 100;         % liczba probek na symbol
  Nmod = 10;          % liczna probek na nosna
  fsym = 1;           % liczba symboli na sekunde
  Kramek = 4;         % liczba wyslanych ramek/blokow symboli
  snr = 60;           % stosunek sygnalu do szumu
  
  fpr = Nsym*fsym;    % czestotliwsc probkowania 
  Nsym = fpr/fsym;    % liczba probek sygnalu na jeden symbol
  
  Npsf = 1001;        % liczba wspolczynnikow filtra ksztaltujacego impuls (PSF)
  ipsf = 1;           % 0/1 = NIE/TAK filtr PSF w odbiorniku
  
  wzm = 0.1;          % wzmocnienie kanalu
  faza = pi/8;        % przesuniecie fazowe kanalu
  kanal = 1;          % znieksztalcenie wprowadzane przez kanal: 0/1 NIE/TAK
  korektor = 0;       % korektor kanalu: 0/1 NIE/TAK
  
% Przesylane dane
  dane = [ 0 1 0 2 3 0 3 1 1 2 1 0 2 2 0 1 3 3 0 2 1 1 0 3 0 2 1 1 3 1 0 2 ];
% dane = [ 0 1 2 3 2 1 0 1 2 3 2 1 0 1 2 3 2 1 0 1 2 3 2 1 0 1 2 3 2 1 2 3 1 1 1 2 2 1 3 0 3];
% dane = [ 0 0 1 1 2 2 3 3 0 0 1 1 2 2 3 3 0 0 1 1 2 2 3 3 0 0 1 1 2 2 3 3 ];
% Preambuly synchronizacyjne:  
% synch = [3 1 0 0 3 2 2 1 3 1 0];                   % NTS1
% synch = [1 3 2 2 1 0 0 3 1 3 2];                   % NTS2
  synch = [3 0 0 1 2 1 3 0 3 2 2 1 3 0 0 1 2 1 3] ;  % STS
  
  psf = firrcos( Npsf-1, fsym/2, 0.35, fpr,'rollof', 'sqrt');  % filtr ksztaltujacy PSF (raised cosine)
  plot(psf'); xlabel('n'); title('psf[n]'); grid; pause

% NADAJNIK ##########################################################################

  ramka = [ synch dane ];                       % pojedynczna wyslana ramka symboli 
  wyslane = [ 0 ];
  for k = 1:Kramek; wyslane = [wyslane ramka];  % wyslana sekwencja numerow stanow
  end    
  Ksym = length(ramka);                         % liczba symboli w jednej ramce 
  Nramka = Ksym*Nsym;                           % liczba probek sygnalu na jedna ramke symboli

  IQwyslane = numer2IQ( wyslane );              % numery symboli --> faza
  sygnal = upfirdn( IQwyslane, psf, Nsym, 1 );  % nadprobkowanie Nsym razy z uzyciem psf
  sygnal = sygnal - mean(sygnal);               % odjecie sredniej
  sygnal = sygnal / std(sygnal);                % normalizacja
  Nsygnal = length(sygnal);

  L = 10000;                                    % ile probek sygnalu pokazac
  figure  % eye-diagram: imag=f(real)
  plot(sygnal(1:L),'b-'); title('I=f(Q)'); pause
  figure  % real, imag
  subplot(211); plot(real(sygnal(1:L))); title('real(IQ)');
  subplot(212); plot(imag(sygnal(1:L))); title('imag(IQ)');

% Modulacja - przejscie na wyzsza czestotliwosc
  n=0:Nsygnal-1;
  sygnal = real(sygnal).*cos(2*pi/Nmod*n) - imag(sygnal).*sin(2*pi/Nmod*n);
   
% KANAL ############################################################################

% Transmisja: splot z odpowiedzia impulsowa kanalu, dodawanie zaklocen
  if(kanal==1) sygnal = sygnal*wzm*exp(-j*faza); end  % tlumienie i opoznienie
  sygnal = awgn( sygnal, snr );                       % szum AWGN
  
% ODBIORNIK ########################################################################

% Demodulacja - powrot na niska czestotliwosc, eliminacja wplywu kanalu
  sreal = cos(2*pi/Nmod*n) .* sygnal;
  simag = -sin(2*pi/Nmod*n) .* sygnal;
  sygnal = sreal + j*simag;
    
% Korektor FEQ  
  if(kanal==1 & korektor==1) sygnal= sygnal*(1/wzm)*exp(j*faza); end

% Rysunek  
  figure     % eye-diagram: imag=f(real)
  plot(sygnal(1:L),'b-'); title('I=f(Q) demod'); pause

% Generacja sygnalu synchronizacji
  IQsynch = numer2IQ( synch );               % numery symboli --> faza
  sync = upfirdn( IQsynch, psf, Nsym, 1 );   % nadprobkowanie Nsym razy z uzyciem psf
  sync = sync - mean(sync);                  % odejecie sredniej
  sync = sync / std(sync);                   % normalizacja
  Nsync = length(sync);
  
% SPLOT Z FILTREM KSZTALTUJACYM IMPULS
  Nstart = (Npsf+1)/2;
  if(ipsf==1)
     sygnal= conv(sygnal,psf); Nsygnal = length(sygnal);
     sync = conv(sync,psf); Nsync = length(sync);
     Nstart = Npsf;  % aktualizacja przesuniecia analizy
  end
  sygnal = sygnal - mean(sygnal);  sync = sync - mean(sync);  % odejecie sredniej
  sygnal = sygnal / std(sygnal);   sync = sync / std(sync);   % normalizacja

% KORELACJA
  korel = abs( conv( fliplr(conj(sync)), sygnal) ); % korelacja sygnalu danych i synchronizacji
  [xmax, imax] = max( korel( 1:round(Nramka) ) );   % znalezienie wartosci i polozenia pierwszego maksimum
  korel = korel / max( korel );                     % normalizacja

  iwork = Nstart+round(Nsym);
  iback = (Nsync+round(Nsym))-iwork;
  nfirst = imax-iback-1;                            % probka srodka pierszego symbolu 

  figure
  n = nfirst + (0 : Ksym*Kramek-1)*Nsym;
  plot( real(sygnal), imag(sygnal),'b-', real(sygnal(n)), imag(sygnal(n)),'ro'); title('I=f(Q)'); pause

  for k = 0 : Kramek-1
     k
     detekt = [];
     num = nfirst + k*Nramka + round((0:Ksym)*Nsym);  % przyblizone polozenia probek kolejnych symboli (DO POPRAWY!)
     fi = atan2(imag(sygnal(num)),real(sygnal(num)));
     dfi = fi(2:end) - fi(1:end-1);
     for m=1:length(dfi)
        if( cos(dfi(m))>0 & sin(dfi(m))>0 )  detekt = [detekt 0]; end
        if( cos(dfi(m))<0 & sin(dfi(m))>0 )  detekt = [detekt 1]; end
        if( cos(dfi(m))>0 & sin(dfi(m))<0 )  detekt = [detekt 2]; end
        if( cos(dfi(m))<0 & sin(dfi(m))<0 )  detekt = [detekt 3]; end  
        if( (sin(dfi(m))==0) | (cos(dfi(m))==0) ) disp('ZERO'); pause; end
     end
     ramka, detekt, pause
     if( sum( abs( ramka - detekt ) ) == 0 ) disp('TO SAMO!!!'); pause; end
  end
