%% Przyklad 22.2 - #1. Transmisja radiowa
%% Nadajnik

% Koder modulatora
M = 4; % wartosciowosc modulacji
SigLen = 4000; % dlugosc sekwencji
Fc = 4e6; % czestotliwosc nosna (Hz)
SymRate = 1e6; % liczba symboli na sekunde
nSamps = 16; % Liczba probek na symbol
Fs = SymRate * nSamps; % czestotliwosc probkowania
EbNo = 20; % stosunek energii przypadajacej na jeden bit do widmowej
% gestosci mocy szumu

% Obliczanie SNR (dla funkcji awgn()), 10*log10(log2(M)/nSamps) bo
% nadprobkowywujemy, 10*log10(2) bo N0/2
SNR = EbNo + 10*log10(log2(M)/nSamps) + 10*log10(2);
msg = randi([0, 1], SigLen, 1); % Zrodlo danych

% Blok modulacji (pasmo podstawowe). W przykladnie z rozdzialu 22.6
% jest on zaimplementowany bez uzycia funkcji modulate()
hMod = modem.pskmod('M', M, 'SymbolOrder', 'gray', 'InputType', 'bit');
% Koder modulatora
TXsig = modulate(hMod, msg); % modulacja

%% Przyklad 22.2 - #2 Ksztaltowanie impulsow
% Parametry filtra pierwiastek z podniesionego kosinusa
FltLength = 8; % rzad filtra
RolloffFactor = 0.22; % wspolczynnik opadania zbocza charakterystyki
% czestotl. (rolloff factor)
filterSpec = fdesign.pulseshaping(nSamps, 'Square root raised cosine', ...
    'Nsym,Beta', FltLength, RolloffFactor);
ImpShapeFlt = design(filterSpec); % tworzymy filtr (obiekt)
TXsigUpsampled = upsample(TXsig, nSamps);   % nadprobkowanie sygnalu
                                            % z kodera modulacji
TXshaped = filter(ImpShapeFlt, TXsigUpsampled); % filtracja

% Widmo
Hamming256 = spectrum.welch('Hamming', 256);
BasebandSpectrum = figure;
psd(Hamming256, TXshaped, 'Fs', Fs, 'SpectrumType', ...
    'twosided','CenterDC', true);

%% Przyklad 22.2 - #3 Konwersja do czestotliwosci radiowej z wzoru Eulera
t = (0:1/Fs:(SigLen/(sqrt(M)*SymRate))-1/Fs).'; % czas
Carrier = sqrt(2)*exp(1i*2*pi*Fc*t); % nosna
TXup = real(TXshaped.*Carrier); % konwersja
PassbandSpectrum = figure;
psd(Hamming256, TXup, 'Fs', Fs, 'SpectrumType','twosided', ...
    'CenterDC', true);

%% Przyklad 22.2 - #4 Kanal AWGN
RXup = awgn(TXup, SNR, 'measured');

% Wyswietlanie widma
figure(PassbandSpectrum);
hold on;
psdProbe = psd(Hamming256, RXup, 'Fs', Fs, 'SpectrumType', ...
    'twosided', 'CenterDC', true);
plot2 = plot(psdProbe);
set (plot2, 'Color','r');
legend('Sygnal na wejsciu kanalu', 'Sygnal na wyjsciu kanalu', ...
    'Location', 'southwest');
hold off;

%% Przyklad 22.2 - #5
% Odbiornik
% Konwerter w dol
RXdown = RXup.*conj(Carrier);
% Wyswietlanie widma
ReceivePlot = figure;
psd(Hamming256, RXdown, 'Fs', Fs, 'SpectrumType', 'twosided', ...
    'CenterDC', true);

%% Przyklad 22.2 - #6
% Filtr odbiorczy
ReceiveFlt = copy(ImpShapeFlt);
RXsym = filter(ReceiveFlt, RXdown);
% Wyswietlanie widma
hold on;
psdProbe = psd(Hamming256, RXsym, 'Fs', Fs, 'SpectrumType', 'twosided',...
    'CenterDC', true);
plot3 = plot(psdProbe);
set (plot3, 'Color','r');
legend('Sygnal przed filtracja',...
    'Sygnal po filtracji', 'Location', 'southwest')
hold off;

%% Przyklad 22.2 - #7
% Detekcja
% Odrzucamy pierwsze FltLength symbole (opoznienie filtra)
FltDelay = FltLength * nSamps;

RXdown = downsample(RXsym(FltDelay+1:end), nSamps); % decymacja
hDemod = modem.pskdemod(hMod); % demodulator (obiekt demod)
RXmsg = demodulate(hDemod, RXdown); % detekcja

liczba_bledow = biterr(msg(1:end-FltLength*log2(M)), RXmsg); % liczba bledow