%% Przyklad 22.3 -- Transmisja radiowa w pasmie podstawowym w kanale AWGN
%% Parametry poczatkowe
siglen = 100000; % dlugosc sekwencji
EbNomin = 0; EbNomax = 10; % Zakres EbNo
EbNovec = EbNomin:2:EbNomax; % Wektor EbNo
numEbNos = length(EbNovec); % Liczba punktow EbNo

% Modulatory
QPSKmod = modem.pskmod('M', 4, 'SymbolOrder', 'gray'); % Modulator QPSK
QPSKdemod = modem.pskdemod(QPSKmod); % Demodulator
numerrminQPSK = 20; % Minimalna liczba bledow aby skonczyc symulacje
berQPSK = zeros(1,numEbNos); % BERy
serQPSK = zeros(1,numEbNos); % SERy
intv4B = cell(1,numEbNos);  % przedzialy ufnosci QPSK

QAM16mod = modem.qammod('M', 16, 'SymbolOrder', 'gray'); % Modulator 16QAM
QAM16demod = modem.qamdemod(QAM16mod); % Demodulator 16QAM
numerrmin16QAM = 20; % Minimalna liczba bledow aby skonczyc symulacje
ber16QAM = zeros(1,numEbNos); % BERy
ser16QAM = zeros(1,numEbNos); % SERy
intv16B = cell(1,numEbNos); % przedzialy ufnosci 16QAM

% Symulacje puszczamy w petli, kazda iteracja dla danej wartosci EbN0.
%% QPSK
for jj = 1:numEbNos
    EbNo = EbNovec(jj);
    
    snr = EbNo + 10*log10(log2(4));
    ntrialsQPSK = 0; % Liczba iteracji petli
    numerrQPSK = 0; % Liczba bledow, dla danej wartoeci EbN0 (bity)
    numSymerrQPSK = 0; % Symbole
    
    % Symuluj tak dlugo, az nie wygeneruje sie okreslona liczba bledow.
    while (numerrQPSK < numerrminQPSK)
        msg = randi([0, 3], siglen, 1);  % Generacja danych
        txsig = modulate(QPSKmod, msg); % Modulacja
        rxsig = awgn(txsig, snr, 'measured'); % Dodanie szumu bialego (kanal awgn)
        decodmsg = demodulate(QPSKdemod, rxsig); % Demodulacja
        newerrsQPSK = biterr(msg, decodmsg); % Liczba bledow w tej iteracji: bity,
        newSymerrsQPSK = symerr(msg, decodmsg); % symbole
        numerrQPSK = numerrQPSK + newerrsQPSK; % calkowita liczba bledow
        numSymerrQPSK = numSymerrQPSK + newSymerrsQPSK;
        ntrialsQPSK = ntrialsQPSK + 1; % Aktualizacja liczby iteracji petli.
    end
    
    % Obliczanie BER
    [berQPSK(jj), intv4B{jj}] = berconfint(numerrQPSK,(ntrialsQPSK * siglen * log2(4)),.95);
    [serQPSK(jj), intv4S] = berconfint(newSymerrsQPSK,(ntrialsQPSK * siglen),.95);
    disp(['QPSK: EbNo = ' num2str(EbNo) ' dB, ' num2str(numerrQPSK) ...
        ' bledow, BER = ' num2str(berQPSK(jj)) ', SER = ' num2str(serQPSK(jj))])
end

%% 16QAM
for jj = 1:numEbNos
    EbNo = EbNovec(jj);
    
    snr = EbNo + 10*log10(log2(16));
    ntrials16QAM = 0;
    numerr16QAM = 0;
    numSymerr16QAM = 0;
    
    while (numerr16QAM < numerrmin16QAM)
        msg = randi([0, 15], siglen, 1);
        txsig = modulate(QAM16mod, msg);
        rxsig = awgn(txsig, snr, 'measured');
        decodmsg = demodulate(QAM16demod, rxsig);
        newerrs16QAM = biterr(msg, decodmsg);
        newSymerrs16QAM = symerr(msg, decodmsg);
        numerr16QAM = numerr16QAM + newerrs16QAM;
        numSymerr16QAM = numSymerr16QAM + newSymerrs16QAM;
        ntrials16QAM = ntrials16QAM + 1;
    end
    
    % Obliczanie BER
    [ber16QAM(jj), intv16B{jj}] = berconfint(numerr16QAM,(ntrials16QAM * siglen * log2(16)),.95);
    [ser16QAM(jj), intv16S] = berconfint(numSymerr16QAM,(ntrials16QAM * siglen),.95);
    disp(['16QAM: EbNo = ' num2str(EbNo) ' dB, ' num2str(numerr16QAM) ...
        ' bledow, BER = ' num2str(ber16QAM(jj)) ', SER = ' num2str(ser16QAM(jj))])
end

%% Wykres
% QPSK
plotQPSK = semilogy(EbNovec(:), berQPSK);
set(get(gca,'Xlabel'),'FontSize',10,'FontName','times');
set(get(gca,'Ylabel'),'FontSize',10,'FontName','times');
set(get(gca,'Title'),'FontSize',12,'FontName','times');
set(plotQPSK,'LineWidth',1,'LineStyle','-', 'Color','b');
set(plotQPSK, 'Marker','x', 'MarkerSize',8, 'MarkerEdgeColor', 'b');
grid on;

% 16QAM
hold on
plot16QAM = semilogy(EbNovec(:), ber16QAM);
set(plot16QAM,'LineWidth',1,'LineStyle','-.', 'Color','r');
set(plot16QAM, 'Marker','o', 'MarkerSize',8, 'MarkerEdgeColor', 'r');
hold off

% Narysuj przedzialy ufnosci
hold on;
for jj=1:numEbNos
    semilogy([EbNovec(jj) EbNovec(jj)],intv4B{jj},'g-+');
    semilogy([EbNovec(jj) EbNovec(jj)],intv16B{jj},'g-+');
end
hold off;

xlabel('Eb/N0 [dB]'); ylabel('Prawdopodobienstwo bledu');
legend('QPSK','16QAM', 'Location', 'southwest');