%% Przyklad 22.4 Transmisja radiowa. Model kanalu FNFC z zanikiem
% plaskim i addytywnym szumem bialym
%% Parametry poczatkowe
siglen = 1000; % dlugosc sekwencji
trainlen = 200; % dlugosc ciagu uczacego
M = 4; % wartosciowosc modulacji
hMod = modem.pskmod('M', M, 'SymbolOrder', 'gray', 'InputType', 'bit');
hDemod = modem.pskdemod(hMod); % Demodulator
EbNomin = 0; EbNomax = 16; % Zakres EbNo
numerrmin = 10000; % Minimalna liczba bledow aby skonczyc symulacje
EbNovec = EbNomin:2:EbNomax; % Wektor EbNo
numEbNos = length(EbNovec); % Liczba punktow EbNo
ber = zeros(1,numEbNos); % BERy
intv = cell(1,numEbNos); % przedzialy ufnosci

%% Kanal
Ts = 1e-7;   % czas trwania probki (symbolu)
Fd = 14;    % maksymalna czestotliwosc Dopplera (Hz)

flat_ch = rayleighchan(Ts,Fd);
flat_ch.StoreHistory = 1;

%% Korektor
eqlms = lineareq(1,lms(0.03));

%% Symulacje puszczamy w petli, kazda iteracja dla danej wartosci EbN0.
for jj = 1:numEbNos
   EbNo = EbNovec(jj);
   snr = EbNo + 10*log10(log2(M)); 
   ntrials = 0;
   numerr = 0;
   
   % Symuluj tak dlugo, az nie wygeneruje sie okreslona liczba bledow.
   while (numerr < numerrmin)
      msg = randi([0, 1], siglen, 1);  % Generacja danych.
      txsig = modulate(hMod, msg); % Modulacja.
      
      % Przejscie sygnalu przez kanal FNFC z zanikiem plaskim i efektem Dopplera
      rxsig = awgn(filter(flat_ch,txsig), snr);
      
      % Korekcja
      eqsig = equalize(eqlms,rxsig,txsig(1:trainlen));
      
      decodmsg = demodulate(hDemod, eqsig);
      newerrs = biterr(msg(1:trainlen), decodmsg(1:trainlen), log2(M)); 
      numerr = numerr + newerrs; 
      ntrials = ntrials + 1; 
   end
   
   [ber(jj), intv1] = berconfint(numerr,(ntrials * siglen),.95);
   disp(['EbNo = ' num2str(EbNo) ' dB, ' num2str(numerr) ...
         ' bledow, BER = ' num2str(ber(jj))])
end

%% Wykres
plotBER = semilogy(EbNovec(:), ber);
set(plotBER,'LineWidth',1,'LineStyle','-');
set(plotBER, 'Marker','x');
set(plotBER,'MarkerSize',12, 'MarkerEdgeColor', 'k')
set(plotBER, 'Color','k');
grid on;
% title('BER vs EbN0')
xlabel('SNR [dB]');
ylabel('Prawdopodobienstwo bledu');
