%% Przyklad 22.5 Transmisja radiowa. Model kanalu FSFC z zanikiem selektywnym
% i addytywnym szumem bialym
%% Parametry poczatkowe
siglen = 1000; % dlugosc sekwencji
trainlen = 200; % dlugosc ciagu uczacego
M = 4; % wartosciowosc modulacji
hMod = modem.pskmod('M', M, 'SymbolOrder', 'gray', 'InputType', 'bit'); 
hDemod = modem.pskdemod(hMod);   
EbNomin = 3; EbNomax = 18; % Zakres EbNo
numerrmin = 30000; % Minimalna liczba bledow aby skonczyc symulacje
EbNovec = EbNomin:3:EbNomax; % Wektor EbNo
numEbNos = length(EbNovec); % Liczba punktow EbNo
ber = zeros(1,numEbNos); % BERy
intv = cell(1,numEbNos); % Komorka na przedzialy ufnosci
Tber = zeros(1,numEbNos); % Teoretyczne BERy

%% Kanal
Ts = 2.6e-7;   % czas trwania probki
Fd = 10;    % maksymalna czestotliwosc Dopplera (Hz)

sel_ch = stdchan(Ts, Fd, '3gppTUx');    % obiekt kanalu
sel_ch.NormalizePathGains = 0;
ch_delay = sel_ch.ChannelFilterDelay;
sel_ch.ResetBeforeFiltering = 0;

%% Korektor o rzedzie wiekszym niz 1
nWeights = 30;  % rzad korektora (liczba wspolczynnikow wagowych)
eqrls = lineareq(nWeights, rls(0.98,0.1)); % stworzenie obiektu korektora
eqrls.SigConst =  hMod.Constellation; % ustawienie konstelacji sygnalu
eqrls.ResetBeforeFiltering = 0; 
eqrls.RefTap = floor(nWeights/2);
eq_delay = (eqrls.RefTap -1)/eqrls.nSampPerSym;

%% Symulacje puszczamy w petli, kazda iteracja dla danej wartosci EbN0.
for jj = 1:numEbNos
   EbNo = EbNovec(jj);
   Tber(jj) = berfading(EbNo, 'psk', M, 10);
   snr = EbNo + 10*log10(log2(M)); 
   ntrials = 0;
   numerr = 0;
   
   % Symuluj tak dlugo, az nie wygeneruje sie okreslona liczba bledow.
   while (numerr < numerrmin)
      msg = randi([0, 1], siglen, 1);  % Generacja danych.
      txsig = modulate(hMod, msg); % Modulacja.
      
      % Przejscie sygnalu przez kanal z zanikiem plaskim i efektem Dopplera
      rxsig = awgn(filter(sel_ch,txsig), snr, 'measured'); % Add noise.
      
      % kompensacja opoznienia kanalu (selektywnego czestotliwosciowo)
      msg_trunc = msg(1:end-2*ch_delay);      % opusc ostatnie ch_delay symbole wiadomosci
      txsig_trunc = txsig(1:end-ch_delay);    % oraz sygnalu nadanego
      rxsig_trunc = rxsig(ch_delay+1:end);    % opusc pierwsze ch_delay symbole sygnalu odebranego
      
      % Korekcja
      eqsig = equalize(eqrls, rxsig,txsig(1:trainlen));
      
      % kompensacja opoznienia korektora (tez jest filtrem pamieciowym)
      RXsig = eqsig(eq_delay+1:end); % opusc pierwsze eq_delay symbole odebranego sygnalu
      TXmsg = msg_trunc(1:end - 2*eq_delay);    % opusc ostatnie eq_delay 
                                                % symbole nadanego sygnalu
      msglen = length(TXmsg);
      
      decodmsg = demodulate(hDemod, RXsig); % Demodulacja.
      newerrs = biterr(msg(1:trainlen), decodmsg(1:trainlen), log2(M));
      numerr = numerr + newerrs;
      ntrials = ntrials + 1;
      nBits = ntrials * msglen;
   end
% Obliczanie liczby bledow oraz BER
   [ber(jj), intv1] = berconfint(numerr, nBits,.95);
   disp(['EbNo = ' num2str(EbNo) ' dB, ' num2str(numerr) ...
         ' bledow, BER = ' num2str(ber(jj))])
end

%% Wykres
plotBER = semilogy(EbNovec(:), ber);
set(plotBER,'LineWidth',1,'LineStyle','-');
set(plotBER, 'Marker','+');
set(plotBER,'MarkerSize',12, 'MarkerEdgeColor', 'k')
set(plotBER, 'Color','k');
grid on;
% title('BER vs EbN0')
xlabel('Eb/N0 [dB]');
ylabel('Prawdopodobienstwo bledu');