function [ teq, Dopt ]= AdslTEQ( h, Le, Ld, D )
% h    - odpowiedz impulsowa kanalu)
% Le   - zadana dlugosc filtra skracajacego (korektora) TEQ
% Ld   - zadana dlugosc odpowiedzi impulsowej kanalu po skroceniu
% D    - zadana liczba probek oponienia skroconej odpowiedzi impulsowej
% e    - obliczone wspolczynniki filtra skracajacego
% Dopt - obliczone optymalne opoznienie (opcjonalnie)

Lh = length(h);                   % dlugosc odpowiedzi impulsowej kanalu
H = zeros(Le, Lh+Le-1);
for k=1:Le
    H(k, k:k+Lh-1) = h;           % macierz "splotowa" kanalu
end

Dhmax = find(h==max(abs(h)));     % numer probki, dla ktorej odpowiedz impulsowa ma maksimum
if(D~=0) Dmin=D; Dmax=D;          % nie szukaj lepszego oplnienia dla skroconej odpowiedzi impulsowej
else     Dmin=0; Dmax=Dhmax+Le;   % szukaj optymalnego oponienia dla skroconej odpowiedzi impulsowej
end

E0min=0;                          % petla: poszukiwanie optymalnego opoznienia
for d = 0 : Dmax
    rect = [zeros(1,d),ones(1,Ld),zeros(1,Lh+(Le-1)-Ld-d)]; % prostokatne okno, w ktorym ma sie zmiescila skrocona odpowiedz impulsowa
    W1 = diag(rect);		      % umieszczenie okna na przekatnej macierzy
    W0 = diag(1-rect);	          % umieszczenie jego "odwrocenia" (1 -> 0, 0 -> 1) na przekatnej macierzy
    X1  = H*W1*H';
    G   = chol(X1);
    X0 = H*W0*H';
    T  = inv(G)'*X0*inv(G);
    [V,D] = eig(T);
    LambdaMin = min(diag(D));
    LambdaMinIdx = find(diag(D)==LambdaMin);
    e = V(:,LambdaMinIdx)' * inv(G)';
    E0 = e*X0*e';
	 if((E0min==0) | (E0 < E0min))
		 E0min = E0;
       emin = e;
       Dopt = d;
	 end
end
teq = emin;
