% Przyklad 23.2 - plik adsl.m
function y = adsl( iter, trType, bitMask, teqIR, Dx, feq, noiseType )
% iter: liczba iteracji (ramek)
% trType: typ transmisji:
% 0 = sygnal treningowy {1,2,3}, bez prefiksu, bez TEQ, bez FEQ, bitMask ignorowany
% 1 = estymacja SNR: jak dla 0 plus prefiks, TEQ, FEQ, bitMask ignorowany
% 2 = transmisja danych: wszystkie elementy modemu dzialaja
% bitMask: (N/2,1) liczba bitow transmitowany w podkanale, 0 - brak transmisji

Lp = 32;                    % dlugosc prefiksu
N = 512;                    % dlugosc ramki bez prefiksu
fp = 2.208e6;               % czestotliwosc probkowania
sigPower = 23;              % moc sygnalu [dBm]
% noiseType = 'AWGN';       % %typ zaklocen addytywnych (szumu): AWGN, NBI, impulsowy, etc...
noisePower = -100;          % moc szumu AWGN [dBm/Hz]
noisePowerNBI = -20;        % moc szumu NBI [dBm]
noiseChannelNBI = 100;      % numer podkanalu w ktorym wystapilo zaklocenie NBI
channelNr = 2;              % numer kanalu z bazy csaloop: do wyboru {1...8, 10, 11}
H = zeros( 1, N/2 );
sig = zeros( 1, N/2 );
noise = zeros( 1, N/2 );
ber = zeros( 1, N/2 );

% glowna petla programu - jedna iteracja = transmisja jednej ramki danych
for i = 1:iter
    X1 = genBinData( trType, bitMask );
    X2 = QAM( X1, trType, bitMask );
    x3 = Mod( X2, N );
    x4 = Prefix( x3, trType, Lp );
    x5 = NormEnergy( x4, sigPower );
    y1 = channel( x5, channelNr );
    y2 = AddNoise( y1, noiseType, noisePower, noisePowerNBI, noiseChannelNBI, fp, iter );
    y3 = DeNormEnergy( y2, sigPower );
    y4 = TEQ( y3, teqIR );
    y5 = DePrefix( y4, trType, N, Dx, Lp );
    Y6 = DeMod( y5, N );
    Y7 = FEQ( Y6, feq );
    Y8 = DeQAM( Y7, trType, bitMask );
    H = HEstim( i, trType, X2, Y6, H );
    [sig, noise] = SNR( i, trType, X2, Y7, sig, noise );
    ber = BER( i, trType, bitMask, X1, Y8, ber );
    %     err = cmp( X1, Y8, X2, Y7 );
end

if( trType == 0 )                               % estymacja knanalu
    H( N/2+2:N ) = conj( fliplr( H(2:N/2) ) );
    H = H / (iter-1);
    h = ifft( H );
    h = real(h);
    y = h;
elseif( trType == 1 )                           % obliczanie SNR
    snr = sig./noise;
    y = snr;
elseif( trType == 2 )                           % transmisja bitow
    y = ber;
end

%===================== genBinData ====================
function y = genBinData( trType, mask )
% trType: typ transmisji:
%   0 = sygnal treningowy {1,2,3,4}, bez: prefiksu, TEQ, FEQ; bitMask ignorowany
%   1 = esytmacja SNR: wygnal jak w 0, uwzgledniono: prefiks, TEQ, FEQ, bitMask ignorowany
%   2 = transmisja danych
% mask(N/2,1): ilosc bitol w kazdy podpasmie
% y(N/2,1): wektor wyjsciowy zawiarajacy losowe dane
persistent state;
if isempty( state )  % zapewnia brak korelacji pomiedzy szumem danymi (losowymi)
    rand( 'state', 111 );
    disp( 'initialize persistent random data generator' );
else
    rand( 'state', state );
end

if( trType == 0 || trType == 1 )  % ReverbSequence (ADSL norma C-REVERB1 s.92)
    N = 512;
    b = zeros( N, 1 );
    b( 1:9 ) = 1;
    for i =10:length(b)
        b(i) = xor( b(i-4), b(i-9) );
    end
    
    for i = 1:N/2
        y( i ) = ( 2 * b( 2*(i-1)+1 ) + b( 2*(i-1)+2 ) ) +1;
    end
elseif( trType == 2 )
    for s=1:length(mask)
        if( mask(s) == 0 )
            y(s) = 0;
        else
            y(s) = randint( 1, 1, 2^mask(s) );
        end
    end
else
    y = zeros( size(mask) );
end
state = rand( 'state' );


%===================== QAM ====================
function y = QAM( x, trType, mask )
% x(N/2,1): dane binarne {0, 1, 2, 3, 4, 5, ... 2^n-1}
% mask(N/2,1): liczba bitow w kazdym podkanale
% y(N/2,1): QAM
y=zeros( size(mask) );
if( trType == 0 || trType == 1 )    % estymacja SNR i kanalu, uzywane wylacznie QAM4
    QAM4 = [ 1+j, 1-j; -1+j, -1-j ];
    y = zeros( size( mask ) );
    for i = 1:length( mask )
        if( mask(i) ~= 2 )
            y(i) = 0;
        else
            y(i) = QAM4( x(i) );
        end
    end
elseif( trType == 2 )       % QAM4, QAM8, ..., QAM32768
    qamMax = [ 0 1 3 3 5 7 11 15 23 31 47 63 95 127 191 ];
    for s=1:length(mask)
        M = 2^mask(s);
        if( mask( s ) == 0 )
            y(s) = 0;
        else
            y(s) = qammod( x(s), M ) / qamMax( mask(s) );
        end
    end
else
    y = zeros( size(mask) );
end


%===================== Mod ====================
function y = Mod( x, N )
% x(N/2,1): QAM
% N: dlugosc ramki (512 dla adsl)
% y(N,1): dane zmodulowane (ifft dla adsl)
% uwaga: zmienna y nie moze zawierac czesci urojonej
%        sygnal wyjsciowy musi byc unormowany do energii jednostkowej

x(1) = sqrt( 2 );
x( N/2+1 ) = sqrt( 2 );
% x(1) = 0;
% x( N/2+1 ) = 0;

x( N/2+2: N ) = conj( fliplr( x(2:N/2 ) ) );
y = ifft( x );
y = y.*sqrt(N/2);


%===================== prefix ====================
function y = Prefix( x, trType, Lp )
% x(N,1): dane zmodulowane (w dziedzinie czasu)
% trType: typ transmisji:
%   0: bez: prefiksu, TEQ, FEQ; bitMask ignorowany
%   1,2: transmisja, wymagany przefiks
if( trType == 0 )
    y = x;
else
    y = [ x(end-Lp+1:end), x ];
end


%===================== NormEnergy ===============
function y = NormEnergy( x, power )
% x(N+Lp,1): jedna ramka danych w dziedzinie czasu
% power: moc syngnalu w dBm
inputImp = 100;
signalPower = inputImp * 0.001 * 10^( power/10 );
y = x .* sqrt( signalPower );


%===================== channel ==================
function y = channel( x, nr )
% x(N+Lp,1): jedna ramka sygnalu przed kanalem
% nr: numer symulowanego kanalu {1...8, 10, 11}
% y(N+Lp,1): jedna ramka sygnalu po przejsciu przez kanal
persistent ir_residue;
persistent ir;
if isempty( ir_residue )
    load ir;                % ladowanie odpowiedzi impulsowych roznych kanalow
    eval( sprintf( 'ir=ir%d;', nr ) );
    ir_residue = zeros( length( ir ) -1, 1 );
    disp( 'initialize persistent ir_residue array' );
end
[ y ir_residue ] = filter( ir, 1, x, ir_residue );


%===================== noise ===================
function y = AddNoise( x, type, power, powerNBI, channelNBI, fs, iter )
% x(N+Lp,1)
% type: typ szumu: {AWGN, NBI, Impulse}
% power: moc szumu w dBm/Hz
% fp: czestotliwosc pro"kowania
persistent state;
persistent NBI;
if isempty( state )   % zapewnia brak korelacji pomiedzy szumem danymi (losowymi)
    rand( 'state', 112 );
    disp( 'initialize persistent random noise' );
    NBI = narrowNoiseGen( fs, iter*544, channelNBI/256/2*fs, 'AM-FIR',  fs/512, 12345 )';
else
    rand( 'state', state );
end
y = x;
if( strcmp(type,'AWGN') || strcmp( type, 'AWGN+NBI' ) )
    inputImp = 100;
    npower = inputImp * 0.001 * fs/2 * 10^( power/10 );
    xn = randn( size(x) );
    y = y + xn .* sqrt( npower );
end
if( strcmp( type, 'NBI' ) || strcmp ( type, 'AWGN+NBI' ) )
    inputImp = 100;
    noiseNotchGain = inputImp * 0.001 * 10^( powerNBI/10 );
    y = y + NBI( 1+(iter-1)*length(x) : iter*length(x) ) * sqrt( noiseNotchGain );
end
if( strcmp( type, 'Impulse' ) )
    error( 'AddNoise - Impulse noise: missing code' );
end
state = rand( 'state' );


%===================== NormEnergy ===============
function y = DeNormEnergy( x, power )
% x(N+Lp,1)
% y(N+Lp,1)
inputImp = 100;
signalPower = inputImp * 0.001 * 10^( power/10 );
y = x ./ sqrt( signalPower );


%===================== TEQ ====================
function y = TEQ( x, ir )
% x(N+Lp,1)
% ir(Le,1): odpowiedz impulsowa korektora TEQ
% y(N+Lp,1): ramka poddana korekcji TEQ w dziedzinie czasu
persistent eq_residue;
if isempty( eq_residue )
    eq_residue = zeros( length( ir ) -1, 1 );
    disp( 'initialize persistent eq_residue array' );
end
[ y eq_residue ] = filter( ir, 1, x, eq_residue );


%===================== DePrefix =================
function y = DePrefix( x, trType, N, D, Lp )
% x(N+Lp,1)
% type: typ transmisji
%     0: sekwencja treningowa - bez prefiksu
%     1: transmisjia - prefiks wymagany
% N: liczba probek w ramce (dlugosc ramki w dziedzinie czasu)
% D: opoznienie transmisji powodowane przez kanal i TEQ
% Lp: dlugosc prefiksu
% y(N,1): jedna ramka danych w dziedzinie czasu bez prefiksu
persistent lastFrame;
if isempty( lastFrame )
    lastFrame = zeros( size( x ) );
    disp( 'initialize last frame delay' );
end

cx = [ lastFrame, x ];
lastFrame = x;

if( trType == 0 )
    y = x;
else
    y = cx( D+Lp+1 : N+D+Lp );
end

%===================== DeMod ==================
function y = DeMod( x, length )
% x(N,1)
% length: dlugosc ramki w dziedzinie czasu
% y(N/2,1): zespolone dane w dziedzinie czestotliwosci (zdemodulowane)
y = fft( x );
y = y( 1:length/2 );
y = y ./ sqrt( length/2 );


%===================== FEQ ====================
function y = FEQ( x, feq )
% x(N/2,1): dane zespolone
% feq(N/2,1): korektor czestotliwosciowy - FEQ
y = x .* feq;


%===================== DeQAM ==================
function y = DeQAM( x, trType, mask )
% x(N/2,1): dane po korekcji FEQ - dziedzina czestotliwosci
% mask(N/2,1): liczba bitow w podkanalach
% y(N/2,1): dane binarne

if( trType == 0 || trType == 1 )
    %QAM4 = [ 1+j, 1-j; -1+j, -1-j ];
    x1 = round( x );
    for i = 1:length( x )
        rx = real( x1(i) );
        ix = imag( x1(i) );
        if( rx==1 && ix==1 )
            y(i) = 1;
        elseif( rx==1 && ix==-1 )
            y(i) = 2;
        elseif( rx==-1 && ix==1 )
            y(i) = 3;
        elseif( rx==-1 && ix==-1 )
            y(i) = 4;
        else
            y(i) = 0;
        end
    end
elseif( trType == 2 )
    qamMax = [ 0 1 3 3 5 7 11 15 23 31 47 63 95 127 191 ];
    for s=1:length(mask)
        M = 2^mask(s);
        if( mask( s ) == 0 )
            y(s) = 0;
        else
            y(s) = qamdemod( x(s)*qamMax( mask(s) ), M );
        end
    end
else
    y=zeros( size(mask) );
end


%===================== H estimation ===============
function H = HEstim( i, trType, X, Y, Hp )
% estymacja odpowiedzi impulsowej, pierwsza ramka jest bledna, nalezy ja zignorowac
% iter: numer iteracji (przeslanej ramki)
% type: typ transmisji
%     0: sekwencja treningowa = identyfikacja kanalu
%     1: estymacja SNR
%     2: transmisjia bitow
% X(N/2,1): transmitowane dane w dziedzinie czestotliwosci
% Y(N/2,1): odebrane dane w dziedzinie czestotliwosci
% Hp(N/2,1): odpowiedz czestotliwosciowa kanalu z poprzedniej ramki
% H(N/2,1): odpowiedz czestotliwosciowa kanalu - usrednianie
if( trType ~= 0 )
    H = Hp;
    return;
else
    if( i == 1 )
        H = zeros( size(Hp) );
    else
        H = Hp + Y./X;
    end
end

function [sig, noise] = SNR( i, trType, X, Y, sigPrev, noisePrev )
% estymacja SNR (Signal-to-Noise-Ratio), pierwsza ramka jest bledna, nalezy ja zignorowac
% iter: iteration number
% iter: numer iteracji (przeslanej ramki)
% type: typ transmisji
%     0: sekwencja treningowa = identyfikacja kanalu
%     1: estymacja SNR
%     2: transmisjia bitow
% X(N/2,1): transmitowane dane w dziedzinie czestotliwosci
% Y(N/2,1): odebrane dane w dziedzinie czestotliwosci
% sigPrev: moc sygnalu z poprzedniej ramki
% noisePrev: moc szumol z poprzedniej ramki
% sig: moc sygnalu - usrednianie
% noise: moc szumu - usrednianie
if( trType ~=1 )
    sig = zeros( size( sigPrev ) );
    noise = zeros( size( noisePrev ) );
    return;
else
    if( i == 1)                         % pierwsza ramka
        sig = zeros( size( sigPrev ) );
        noise = zeros( size( noisePrev ) );
    else
        e = Y - X;
        e = e.*conj(e);
        
        sig = sigPrev + (X.*conj(X));
        noise = noisePrev + e;
    end
end


function ber = BER( i, trType, mask, X, Y, berp )
% estymacja BER, pierwsza ramka jest bledna, nalezy ja zignorowac
% type: typ transmisji
%     0: sekwencja treningowa = identyfikacja kanalu
%     1: estymacja SNR
%     2: transmisjia bitow
% mask: liczba bitow w podkanalach
% X(N/2,1): tansmitowane dane (binarnie)
% Y(N/2,1): odebrane dane (binarnie)
persistent lastX;
if isempty( lastX )
    lastX = zeros( size( X ) );
    disp( 'initialize last frame delay' );
end

if( trType ~=2 )
    ber = zeros( size(X) );
    return;
else
    %error( 'BER: missing code' );
    if( i<2 )
        ber = zeros( size(X) );
    else
        ber = berp + ( (lastX-Y) ~= 0 );
    end
end
lastX = X;

%===================== cmp ====================
function err = cmp( x1, y1, x2, y2 )
% porownywanie, wykresy, etc...
persistent lastX2;
if isempty( lastX2 )
    lastX2 = zeros( size( x2 ) );
    disp( 'initialize last frame delay' );
    err =  zeros( size(x2) );
    return;
end

figure(1);
t = 1:length(y2);
plot( t, real( lastX2(t)), 'b.', t, real(y2(t)), 'bo', t, imag( lastX2(t)), 'rx', t, imag(y2(t)), 'ro' );
err = zeros( size(x1) );
lastX2 = x2;


