% Przyklad 23.1 - ilustracja transmisji ADSL zgodnie ze schematem przedstawionym
% na rysunku 23.4

clear;
fs = 2.208e6; 
codingGain = 4.2; 
margin = 6;

% inicjalizacja parametrow transmisji
N = 512; 
Ld = 32; 
Le = 16; 
Nd = N + Ld; 
Ns = 200;
inputPower = 23; 
inputImp = 100; 
lineNoise = -120;
load c_w_Dx;

% korektor TEQ, IR kanalu transmisyjnego
c = [ c zeros( 1, N-length(c)) ];
sir = conv( w, c );

% skrocona odpowiedz impulsowa kanalu SIR
sir = [ sir zeros(1,N-length(sir)) ];
SIR = fft( sir(1:N ) ).'; k = (0:N-1)';
FEQ = SIR .* (1/(2*sqrt(2))) .* exp( j*2*pi/N *k* Dx );

% wagi korektora FEQ
rb = zeros(2*Nd,1); 
tf_sig = zeros(N,1); 
tf_err = zeros(N,1); % inicjalizacja modemu
U = zeros(N,1); 
U0 = zeros(N,1); 
c_res = zeros( 1, length( c ) -1 );
w_res = zeros( 1, length( w ) -1 );
noiseAWGN = randn(Ns*Nd,1) * sqrt(inputImp*0.001*fs/2*10^(lineNoise/10)); % szumy
signalPower = inputImp * 0.001 * 10^( inputPower/10 ); % wzmocnienie sygnalu
scalingFactor = sqrt( 10^( codingGain/10 ) ) * sqrt( signalPower / 1 );

for iter=1:Ns
    U0 = U;
    Ur=sqrt(3)*(2*rand(N/2-1, 1)-1);
    Ui=sqrt(3)*(2*rand(N/2-1, 1)-1);
    U = Ur+j*Ui;
    U = [ 0; U; 0; conj( flipud(U) ) ];
    x = ifft( U );
    x = ( real( x )*sqrt(N) ) / sqrt(2);
    x = [(x(N-Ld+1:N)); x];
    s = x * scalingFactor;
    
    [ y c_res ] = filter( c, 1, s, c_res );     % kanal
    y = y + noiseAWGN( (iter-1)*Nd+1:iter*Nd ); % dodanie szumu addytywnego
    y = y / scalingFactor;                      % normowanie mocy sygnalu
    [ ye w_res ] = filter( w, 1, y, w_res );    % korektor TEQ
    rb(1:Nd) = rb(Nd+1:2*Nd);
    rb(Nd+1:2*Nd) = ye;
    y = rb(1+Dx+Ld : N+Dx+Ld);                  % usuniecie prefiksu
    Y= fft(y);                                  % blok funkcyjny OFDM
    Y = Y/(2*sqrt(N));                          % normowanie sygnalu
    U_r = Y ./ FEQ;                             % korektor FEQ
    if( iter > 2 )                              % wyznaczanie SNR
        e = U0 - U_r;                           % blad rekonstrukcji
        tf_sig = tf_sig + U0.*conj(U0);         % moc sygnalu
        ce = e .* conj(e);                      
        tf_err = tf_err + ce;                   % moc zaklocen
    end
end

idx = find( tf_err == 0 ); tf_err( idx ) = eps;
snr = tf_sig(1:N/2) ./ tf_err(1:N/2);           % stosunek mocy sygnalu do szumu
